@@ -1,6 +1,6 @@
 /*  bootstrap.c
  *
- *  Copyright (C) 2016-2024 Toxic All Rights Reserved.
+ *  Copyright (C) 2016-2026 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic. Toxic is free software licensed
  *  under the GNU General Public License 3.0.
@@ -344,13 +344,13 @@ static int update_DHT_nodeslist(const Run_Options *run_opts, const char *nodes_p
     return 1;
 }
 
-static void get_nodeslist_path(const Run_Options *run_opts, char *buf, size_t buf_size)
+static void get_nodeslist_path(const Run_Options *run_opts, const Paths *paths, char *buf, size_t buf_size)
 {
     char *config_dir = NULL;
 
     if (!string_is_empty(run_opts->nodes_path)) {
         snprintf(buf, buf_size, "%s", run_opts->nodes_path);
-    } else if ((config_dir = get_user_config_dir()) != NULL) {
+    } else if ((config_dir = get_user_config_dir(paths)) != NULL) {
         snprintf(buf, buf_size, "%s%s%s", config_dir, CONFIGDIR, DEFAULT_NODES_FILENAME);
         free(config_dir);
     } else {
@@ -507,8 +507,8 @@ static void *load_nodeslist_thread(void *data)
         goto on_exit;
     }
 
-    char nodes_path[PATH_MAX];
-    get_nodeslist_path(toxic->run_opts, nodes_path, sizeof(nodes_path));
+    char nodes_path[TOXIC_MAX_PATH_LENGTH];
+    get_nodeslist_path(toxic->run_opts, toxic->paths, nodes_path, sizeof(nodes_path));
 
     FILE *fp = NULL;
 
@@ -667,11 +667,10 @@ void do_tox_connection(Toxic *toxic)
         return;
     }
 
-    static time_t last_bootstrap_time = 0;  // TODO: Put this in Toxic
     const bool connected = prompt_selfConnectionStatus(toxic) != TOX_CONNECTION_NONE;
 
-    if (!connected && timed_out(last_bootstrap_time, TRY_BOOTSTRAP_INTERVAL)) {
+    if (!connected && timed_out(toxic->last_bootstrap_time, TRY_BOOTSTRAP_INTERVAL)) {
         DHT_bootstrap(toxic->tox);
-        last_bootstrap_time = get_unix_time();
+        toxic->last_bootstrap_time = get_unix_time();
     }
 }
