@@ -1,6 +1,6 @@
 /*  chat.c
  *
- *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
+ *  Copyright (C) 2014-2026 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic. Toxic is free software licensed
  *  under the GNU General Public License 3.0.
@@ -161,7 +161,7 @@ void kill_chat_window(ToxWindow *self, Toxic *toxic)
     free(self->help);
     free(statusbar);
 
-    disable_friend_window(self->num);
+    disable_friend_window(toxic->friends, self->num);
     kill_notifs(self->active_box);
     del_window(self, toxic->windows, toxic->c_config);
 }
@@ -215,7 +215,7 @@ static void chat_onMessage(ToxWindow *self, Toxic *toxic, uint32_t num, Tox_Mess
     }
 
     char name[TOXIC_MAX_NAME_LENGTH + 1];
-    get_friend_name(name, sizeof(name), num);
+    get_friend_name(toxic->friends, name, sizeof(name), num);
 
     if (type == TOX_MESSAGE_TYPE_NORMAL) {
         recv_message_helper(self, toxic, msg, name);
@@ -228,7 +228,7 @@ static void chat_onMessage(ToxWindow *self, Toxic *toxic, uint32_t num, Tox_Mess
     }
 }
 
-static void chat_pause_file_transfers(uint32_t friendnum);
+static void chat_pause_file_transfers(FriendsList *friends, uint32_t friendnum);
 static void chat_resume_file_senders(ToxWindow *self, const Toxic *toxic, uint32_t fnum);
 
 static void chat_onConnectionChange(ToxWindow *self, Toxic *toxic, uint32_t num, Tox_Connection connection_status)
@@ -248,7 +248,7 @@ static void chat_onConnectionChange(ToxWindow *self, Toxic *toxic, uint32_t num,
     const char *msg;
 
     char name[TOXIC_MAX_NAME_LENGTH + 1];
-    get_friend_name(name, sizeof(name), num);
+    get_friend_name(toxic->friends, name, sizeof(name), num);
 
     Tox_Connection prev_status = statusbar->connection;
     statusbar->connection = connection_status;
@@ -263,13 +263,13 @@ static void chat_onConnectionChange(ToxWindow *self, Toxic *toxic, uint32_t num,
             write_to_log(ctx->log, c_config, msg, name, LOG_HINT_CONNECT);
         }
     } else if (connection_status == TOX_CONNECTION_NONE) {
-        Friends.list[num].is_typing = false;
+        toxic->friends->list[num].is_typing = false;
 
         if (self->chatwin->self_is_typing) {
             set_self_typingstatus(self, toxic, false);
         }
 
-        chat_pause_file_transfers(num);
+        chat_pause_file_transfers(toxic->friends, num);
 
         if (c_config->show_connection_msg) {
             msg = "has gone offline";
@@ -291,7 +291,7 @@ static void chat_onTypingChange(ToxWindow *self, Toxic *toxic, uint32_t num, boo
         return;
     }
 
-    Friends.list[num].is_typing = is_typing;
+    toxic->friends->list[num].is_typing = is_typing;
 }
 
 static void chat_onNickChange(ToxWindow *self, Toxic *toxic, uint32_t num, const char *nick, size_t length)
@@ -324,7 +324,7 @@ static void chat_onNickRefresh(ToxWindow *self, Toxic *toxic)
     StatusBar *statusbar = self->stb;
 
     char new_name[TOXIC_MAX_NAME_LENGTH + 1];
-    const uint16_t n_len = get_friend_name(new_name, sizeof(new_name), self->num);
+    const uint16_t n_len = get_friend_name(toxic->friends, new_name, sizeof(new_name), self->num);
 
     if (strcmp(new_name, statusbar->nick) == 0) {
         return;
@@ -335,8 +335,8 @@ static void chat_onNickRefresh(ToxWindow *self, Toxic *toxic)
 
     char other_key[TOX_PUBLIC_KEY_SIZE];
 
-    if (get_friend_public_key(other_key, self->num)) {
-        if (rename_logfile(toxic->windows, toxic->c_config, statusbar->nick, new_name, self_key, other_key,
+    if (get_friend_public_key(toxic->friends, other_key, self->num)) {
+        if (rename_logfile(toxic->windows, toxic->c_config, toxic->paths, statusbar->nick, new_name, self_key, other_key,
                            self->id) != 0) {
             fprintf(stderr, "failed to rename logfile\n");
         }
@@ -361,8 +361,10 @@ static void chat_onStatusChange(ToxWindow *self, Toxic *toxic, uint32_t num, Tox
     statusbar->status = status;
 }
 
-static void chat_onStatusMessageChange(ToxWindow *self, uint32_t num, const char *status, size_t length)
+static void chat_onStatusMessageChange(ToxWindow *self, Toxic *toxic, uint32_t num, const char *status,
+                                       size_t length)
 {
+    UNUSED_VAR(toxic);
     UNUSED_VAR(length);
 
     if (self == NULL) {
@@ -395,9 +397,9 @@ static void chat_onReadReceipt(ToxWindow *self, Toxic *toxic, uint32_t num, uint
 }
 
 /* Stops active file transfers for this friend. Called when a friend goes offline */
-static void chat_pause_file_transfers(uint32_t friendnum)
+static void chat_pause_file_transfers(FriendsList *friends, uint32_t friendnum)
 {
-    ToxicFriend *friend = &Friends.list[friendnum];
+    ToxicFriend *friend = &friends->list[friendnum];
 
     for (size_t i = 0; i < MAX_FILES; ++i) {
         struct FileTransfer *fts = &friend->file_sender[i];
@@ -418,7 +420,7 @@ static void chat_pause_file_transfers(uint32_t friendnum)
 static void chat_resume_file_senders(ToxWindow *self, const Toxic *toxic, uint32_t friendnum)
 {
     for (size_t i = 0; i < MAX_FILES; ++i) {
-        struct FileTransfer *ft = &Friends.list[friendnum].file_sender[i];
+        struct FileTransfer *ft = &toxic->friends->list[friendnum].file_sender[i];
 
         if (ft->state != FILE_TRANSFER_PAUSED || ft->file_type != TOX_FILE_KIND_DATA) {
             continue;
@@ -451,7 +453,7 @@ static void chat_onFileChunkRequest(ToxWindow *self, Toxic *toxic, uint32_t frie
         return;
     }
 
-    struct FileTransfer *ft = get_file_transfer_struct(friendnum, filenumber);
+    struct FileTransfer *ft = get_file_transfer_struct(toxic->friends, friendnum, filenumber);
 
     if (ft == NULL) {
         return;
@@ -530,7 +532,7 @@ static void chat_onFileRecvChunk(ToxWindow *self, Toxic *toxic, uint32_t friendn
         return;
     }
 
-    struct FileTransfer *ft = get_file_transfer_struct(friendnum, filenumber);
+    struct FileTransfer *ft = get_file_transfer_struct(toxic->friends, friendnum, filenumber);
 
     if (ft == NULL) {
         return;
@@ -578,7 +580,7 @@ static void chat_onFileControl(ToxWindow *self, Toxic *toxic, uint32_t friendnum
         return;
     }
 
-    struct FileTransfer *ft = get_file_transfer_struct(friendnum, filenumber);
+    struct FileTransfer *ft = get_file_transfer_struct(toxic->friends, friendnum, filenumber);
 
     if (!ft) {
         return;
@@ -591,6 +593,10 @@ static void chat_onFileControl(ToxWindow *self, Toxic *toxic, uint32_t friendnum
                 line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "File transfer [%zu] for '%s' accepted.",
                               ft->index, ft->file_name);
                 sound_notify(self, toxic, silent, NT_NOFOCUS | c_config->bell_on_filetrans_accept | NT_WNDALERT_2, NULL);
+                char progline[MAX_STR_SIZE];
+                init_progress_bar(progline);
+                line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "%s", progline);
+                ft->line_id = self->chatwin->hst->line_end->id + 2;
             } else if (ft->state == FILE_TRANSFER_PAUSED) {    /* transfer is resumed */
                 ft->state = FILE_TRANSFER_STARTED;
             }
@@ -636,7 +642,7 @@ static bool chat_resume_broken_ft(ToxWindow *self, Toxic *toxic, uint32_t friend
     size_t i;
 
     for (i = 0; i < MAX_FILES; ++i) {
-        ft = &Friends.list[friendnum].file_receiver[i];
+        ft = &toxic->friends->list[friendnum].file_receiver[i];
 
         if (ft->state == FILE_TRANSFER_INACTIVE) {
             continue;
@@ -721,7 +727,8 @@ static void chat_onFileRecv(ToxWindow *self, Toxic *toxic, uint32_t friendnum, u
         return;
     }
 
-    struct FileTransfer *ft = new_file_transfer(self, friendnum, filenumber, FILE_TRANSFER_RECV, TOX_FILE_KIND_DATA);
+    struct FileTransfer *ft = new_file_transfer(toxic->friends, self, friendnum, filenumber, FILE_TRANSFER_RECV,
+                              TOX_FILE_KIND_DATA);
 
     if (ft == NULL) {
         tox_file_control(tox, friendnum, filenumber, TOX_FILE_CONTROL_CANCEL, NULL);
@@ -740,7 +747,7 @@ static void chat_onFileRecv(ToxWindow *self, Toxic *toxic, uint32_t friendnum, u
         return;
     }
 
-    size_t file_path_buf_size = PATH_MAX + name_length + 1;
+    size_t file_path_buf_size = TOXIC_MAX_PATH_LENGTH + name_length + 1;
     char *file_path = malloc(file_path_buf_size);
 
     if (file_path == NULL) {
@@ -768,7 +775,7 @@ static void chat_onFileRecv(ToxWindow *self, Toxic *toxic, uint32_t friendnum, u
     FILE *filecheck = NULL;
     int count = 1;
 
-    while ((filecheck = fopen(file_path, "r")) || file_transfer_recv_path_exists(file_path)) {
+    while ((filecheck = fopen(file_path, "r")) || file_transfer_recv_path_exists(toxic->friends, file_path)) {
         if (filecheck) {
             fclose(filecheck);
         }
@@ -809,7 +816,7 @@ static void chat_onFileRecv(ToxWindow *self, Toxic *toxic, uint32_t friendnum, u
                    &self->active_box, self->name, "Incoming file: %s", filename);
     }
 
-    const bool auto_accept_files = friend_get_auto_accept_files(friendnum);
+    const bool auto_accept_files = friend_get_auto_accept_files(toxic->friends, friendnum);
 
     if (auto_accept_files) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Auto-accepting file transfer %zu", ft->index);
@@ -837,9 +844,7 @@ static void chat_onConferenceInvite(ToxWindow *self, Toxic *toxic, int32_t frien
         return;
     }
 
-    if (Friends.list[friendnumber].conference_invite.key != NULL) {
-        free(Friends.list[friendnumber].conference_invite.key);
-    }
+    free(toxic->friends->list[friendnumber].conference_invite.key);
 
     char *k = malloc(length * sizeof(char));
 
@@ -849,13 +854,13 @@ static void chat_onConferenceInvite(ToxWindow *self, Toxic *toxic, int32_t frien
     }
 
     memcpy(k, conference_pub_key, length);
-    Friends.list[friendnumber].conference_invite.key = k;
-    Friends.list[friendnumber].conference_invite.pending = true;
-    Friends.list[friendnumber].conference_invite.length = length;
-    Friends.list[friendnumber].conference_invite.type = type;
+    toxic->friends->list[friendnumber].conference_invite.key = k;
+    toxic->friends->list[friendnumber].conference_invite.pending = true;
+    toxic->friends->list[friendnumber].conference_invite.length = length;
+    toxic->friends->list[friendnumber].conference_invite.type = type;
 
     char name[TOXIC_MAX_NAME_LENGTH + 1];
-    get_friend_name(name, sizeof(name), friendnumber);
+    get_friend_name(toxic->friends, name, sizeof(name), friendnumber);
 
     const char *description = type == TOX_CONFERENCE_TYPE_AV ? "an audio conference" : "a conference";
 
@@ -887,30 +892,28 @@ static void chat_onGroupInvite(ToxWindow *self, Toxic *toxic, uint32_t friendnum
         return;
     }
 
-    if (Friends.list[friendnumber].group_invite.data) {
-        free(Friends.list[friendnumber].group_invite.data);
-    }
+    free(toxic->friends->list[friendnumber].group_invite.data);
 
-    Friends.list[friendnumber].group_invite.data = malloc(length * sizeof(char));
+    toxic->friends->list[friendnumber].group_invite.data = malloc(length * sizeof(char));
 
-    if (Friends.list[friendnumber].group_invite.data == NULL) {
+    if (toxic->friends->list[friendnumber].group_invite.data == NULL) {
         return;
     }
 
-    memcpy(Friends.list[friendnumber].group_invite.data, invite_data, length);
-    Friends.list[friendnumber].group_invite.length = length;
-
-    sound_notify(self, toxic, generic_message, NT_WNDALERT_2 | c_config->bell_on_invite, NULL);
+    memcpy(toxic->friends->list[friendnumber].group_invite.data, invite_data, length);
+    toxic->friends->list[friendnumber].group_invite.length = length;
 
     char name[TOXIC_MAX_NAME_LENGTH + 1];
-    get_friend_name(name, sizeof(name), friendnumber);
+    get_friend_name(toxic->friends, name, sizeof(name), friendnumber);
+
+    const uint64_t flags = NT_WNDALERT_2 | c_config->bell_on_invite;
 
     if (self->active_box != -1) {
-        box_silent_notify2(self, toxic, NT_WNDALERT_2 | NT_NOFOCUS, self->active_box,
-                           "invites you to join group chat");
+        box_notify2(self, toxic, generic_message, flags, self->active_box,
+                    "You have been invited to a group chat.");
     } else {
-        box_silent_notify(self, toxic, NT_WNDALERT_2 | NT_NOFOCUS, &self->active_box, name,
-                          "invites you to join group chat");
+        box_notify(self, toxic, generic_message, flags, &self->active_box, name,
+                   "You have been invited to a group chat.");
     }
 
     line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "%s has invited you to join group chat \"%s\"",
@@ -962,7 +965,7 @@ static void chat_onGameInvite(ToxWindow *self, Toxic *toxic, uint32_t friend_num
     uint32_t data_length = length - GAME_PACKET_HEADER_SIZE;
 
     if (data_length > 0) {
-        free(Friends.list[friend_number].game_invite.data);
+        free(toxic->friends->list[friend_number].game_invite.data);
 
         uint8_t *buf = calloc(1, data_length);
 
@@ -971,16 +974,16 @@ static void chat_onGameInvite(ToxWindow *self, Toxic *toxic, uint32_t friend_num
         }
 
         memcpy(buf, data + GAME_PACKET_HEADER_SIZE, data_length);
-        Friends.list[friend_number].game_invite.data = buf;
+        toxic->friends->list[friend_number].game_invite.data = buf;
     }
 
-    Friends.list[friend_number].game_invite.type = type;
-    Friends.list[friend_number].game_invite.id = id;
-    Friends.list[friend_number].game_invite.pending = true;
-    Friends.list[friend_number].game_invite.data_length = data_length;
+    toxic->friends->list[friend_number].game_invite.type = type;
+    toxic->friends->list[friend_number].game_invite.id = id;
+    toxic->friends->list[friend_number].game_invite.pending = true;
+    toxic->friends->list[friend_number].game_invite.data_length = data_length;
 
     char name[TOXIC_MAX_NAME_LENGTH + 1];
-    get_friend_name(name, sizeof(name), friend_number);
+    get_friend_name(toxic->friends, name, sizeof(name), friend_number);
 
     if (self->active_box != -1) {
         box_notify2(self, toxic, generic_message, NT_WNDALERT_2 | c_config->bell_on_invite, self->active_box,
@@ -1022,14 +1025,17 @@ static void chat_onInvite(ToxWindow *self, Toxic *toxic, uint32_t friend_number,
     line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
                   "Incoming audio call! Type: \"/answer\" or \"/reject\"");
 
+    uint64_t box_flags = NT_NOFOCUS | NT_WNDALERT_0;
+
     if (self->ringing_sound == -1) {
         sound_notify(self, toxic, call_incoming, NT_LOOP | c_config->bell_on_invite, &self->ringing_sound);
+        box_flags |= NT_NO_INCREMENT;
     }
 
     if (self->active_box != -1) {
-        box_silent_notify2(self, toxic, NT_NOFOCUS | NT_WNDALERT_0, self->active_box, "Incoming audio call!");
+        box_silent_notify2(self, toxic, box_flags, self->active_box, "Incoming audio call!");
     } else {
-        box_silent_notify(self, toxic, NT_NOFOCUS | NT_WNDALERT_0, &self->active_box, self->name, "Incoming audio call!");
+        box_silent_notify(self, toxic, box_flags, &self->active_box, self->name, "Incoming audio call!");
     }
 }
 
@@ -1469,7 +1475,7 @@ static void chat_onDraw(ToxWindow *self, Toxic *toxic)
 
     Tox_Connection connection = statusbar->connection;
     Tox_User_Status status = statusbar->status;
-    const bool is_typing = Friends.list[self->num].is_typing;
+    const bool is_typing = toxic->friends->list[self->num].is_typing;
 
     pthread_mutex_unlock(&Winthread.lock);
 
@@ -1579,7 +1585,11 @@ static void chat_onDraw(ToxWindow *self, Toxic *toxic)
     self->x = x2;
 
     /* Truncate note if it doesn't fit in statusbar */
-    const int maxlen = x2 - getcurx(statusbar->topline) - KEY_IDENT_BYTES - 6;
+    int maxlen = x2 - getcurx(statusbar->topline) - KEY_IDENT_BYTES - 6;
+
+    if (maxlen > TOX_MAX_STATUS_MESSAGE_LENGTH) {
+        maxlen = TOX_MAX_STATUS_MESSAGE_LENGTH;
+    }
 
     pthread_mutex_lock(&Winthread.lock);
     const size_t statusmsg_len = statusbar->statusmsg_len;
@@ -1587,8 +1597,7 @@ static void chat_onDraw(ToxWindow *self, Toxic *toxic)
 
     if (statusmsg_len > maxlen && maxlen >= 3) {
         pthread_mutex_lock(&Winthread.lock);
-        statusbar->statusmsg[maxlen - 3] = '\0';
-        strcat(statusbar->statusmsg, "...");
+        memcpy(statusbar->statusmsg + maxlen - 3, "...", 4);
         statusbar->statusmsg_len = maxlen;
         pthread_mutex_unlock(&Winthread.lock);
     }
@@ -1622,7 +1631,7 @@ static void chat_onDraw(ToxWindow *self, Toxic *toxic)
     wattron(statusbar->topline, COLOR_PAIR(BAR_TEXT));
 
     for (size_t i = 0; i < KEY_IDENT_BYTES / 2; ++i) {
-        wprintw(statusbar->topline, "%02X", Friends.list[self->num].pub_key[i] & 0xff);
+        wprintw(statusbar->topline, "%02X", toxic->friends->list[self->num].pub_key[i] & 0xff);
     }
 
     wattroff(statusbar->topline, COLOR_PAIR(BAR_TEXT));
@@ -1658,7 +1667,7 @@ static void chat_onDraw(ToxWindow *self, Toxic *toxic)
 
     pthread_mutex_lock(&Winthread.lock);
 
-    if (refresh_file_transfer_progress(self, self->num)) {
+    if (refresh_file_transfer_progress(toxic->friends, self, self->num)) {
         flag_interface_refresh();
     }
 
@@ -1675,7 +1684,8 @@ static void chat_init_log(ToxWindow *self, Toxic *toxic, const char *self_nick)
     char myid[TOX_ADDRESS_SIZE];
     tox_self_get_address(tox, (uint8_t *) myid);
 
-    if (log_init(ctx->log, c_config, self_nick, myid, Friends.list[self->num].pub_key, LOG_TYPE_CHAT) != 0) {
+    if (log_init(ctx->log, c_config, toxic->paths, self_nick, myid, toxic->friends->list[self->num].pub_key,
+                 LOG_TYPE_CHAT) != 0) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to initialize chat log.");
         return;
     }
@@ -1684,7 +1694,7 @@ static void chat_init_log(ToxWindow *self, Toxic *toxic, const char *self_nick)
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to load chat history.");
     }
 
-    if (friend_get_logging_enabled(self->num)) {
+    if (friend_get_logging_enabled(toxic->friends, self->num)) {
         if (log_enable(ctx->log) != 0) {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to enable chat log.");
         }
@@ -1714,8 +1724,8 @@ static void chat_onInit(ToxWindow *self, Toxic *toxic)
     /* Init statusbar info */
     StatusBar *statusbar = self->stb;
 
-    statusbar->status = get_friend_status(self->num);
-    statusbar->connection = get_friend_connection_status(self->num);
+    statusbar->status = get_friend_status(toxic->friends, self->num);
+    statusbar->connection = get_friend_connection_status(toxic->friends, self->num);
 
     const size_t s_len = tox_friend_get_status_message_size(tox, self->num, NULL);
 
@@ -1730,7 +1740,7 @@ static void chat_onInit(ToxWindow *self, Toxic *toxic)
     statusbar->statusmsg_len = strlen(statusbar->statusmsg);
 
     char name[TOXIC_MAX_NAME_LENGTH + 1];
-    const uint16_t n_len = get_friend_name(name, sizeof(name), self->num);
+    const uint16_t n_len = get_friend_name(toxic->friends, name, sizeof(name), self->num);
 
     snprintf(statusbar->nick, sizeof(statusbar->nick), "%s", name);
     statusbar->nick_len = n_len;
@@ -1753,11 +1763,11 @@ static void chat_onInit(ToxWindow *self, Toxic *toxic)
 
     line_info_init(ctx->hst);
 
-    const int tab_name_colour = friend_config_get_tab_name_colour(self->num);
+    const int tab_name_colour = friend_config_get_tab_name_colour(toxic->friends, self->num);
     self->colour = tab_name_colour > 0 ? tab_name_colour : WHITE_BAR_FG;
 
-    friend_set_logging_enabled(self->num, friend_config_get_autolog(self->num));
-    friend_set_auto_file_accept(self->num, friend_config_get_auto_accept_files(self->num));
+    friend_set_logging_enabled(toxic->friends, self->num, friend_config_get_autolog(toxic->friends, self->num));
+    friend_set_auto_file_accept(toxic->friends, self->num, friend_config_get_auto_accept_files(toxic->friends, self->num));
 
     chat_init_log(self, toxic, name);
 
@@ -1769,7 +1779,7 @@ static void chat_onInit(ToxWindow *self, Toxic *toxic)
     line_info_print(self, toxic->c_config);
 }
 
-ToxWindow *new_chat(Tox *tox, uint32_t friendnum)
+ToxWindow *new_chat(FriendsList *friends, uint32_t friendnum)
 {
     ToxWindow *ret = calloc(1, sizeof(ToxWindow));
 
@@ -1818,7 +1828,7 @@ ToxWindow *new_chat(Tox *tox, uint32_t friendnum)
     ret->active_box = -1;
 
     char name[TOXIC_MAX_NAME_LENGTH + 1];
-    const uint16_t n_len = get_friend_name(name, sizeof(name), friendnum);
+    const uint16_t n_len = get_friend_name(friends, name, sizeof(name), friendnum);
 
     set_window_title(ret, name, n_len);
 
