@@ -1,6 +1,6 @@
 /*  log.c
  *
- *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
+ *  Copyright (C) 2014-2026 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic. Toxic is free software licensed
  *  under the GNU General Public License 3.0.
@@ -31,8 +31,8 @@
  * Return path length on success.
  * Return -1 if the path is too long.
  */
-static int create_log_path(const Client_Config *c_config, char *dest, int destsize, const char *name,
-                           const char *selfkey, const char *otherkey)
+static int create_log_path(const Client_Config *c_config, const Paths *paths, char *dest, int destsize,
+                           const char *name, const char *selfkey, const char *otherkey)
 {
     if (!valid_nick(name)) {
         name = UNKNOWN_NAME;
@@ -41,27 +41,23 @@ static int create_log_path(const Client_Config *c_config, char *dest, int destsi
     const char *namedash = otherkey ? "-" : "";
     const char *set_path = c_config->chatlogs_path;
 
-    char *user_config_dir = get_user_config_dir();
+    char *user_config_dir = get_user_config_dir(paths);
     int path_len = strlen(name) + strlen(".log") + strlen("-") + strlen(namedash);
-    path_len += strlen(set_path) ? *set_path : strlen(user_config_dir) + strlen(LOGDIR);
+    path_len += strlen(set_path) ? strlen(set_path) : strlen(user_config_dir) + strlen(LOGDIR);
 
     /* first 6 bytes of selfkey */
     char self_id[32] = {0};
     path_len += KEY_IDENT_BYTES;
-    sprintf(&self_id[0], "%02X", selfkey[0] & 0xff);
-    sprintf(&self_id[2], "%02X", selfkey[1] & 0xff);
-    sprintf(&self_id[4], "%02X", selfkey[2] & 0xff);
-    self_id[KEY_IDENT_BYTES] = '\0';
+    snprintf(self_id, sizeof(self_id), "%02X%02X%02X",
+             selfkey[0] & 0xff, selfkey[1] & 0xff, selfkey[2] & 0xff);
 
     char other_id[32] = {0};
 
     if (otherkey) {
         /* first 6 bytes of otherkey */
         path_len += KEY_IDENT_BYTES;
-        sprintf(&other_id[0], "%02X", otherkey[0] & 0xff);
-        sprintf(&other_id[2], "%02X", otherkey[1] & 0xff);
-        sprintf(&other_id[4], "%02X", otherkey[2] & 0xff);
-        other_id[KEY_IDENT_BYTES] = '\0';
+        snprintf(other_id, sizeof(other_id), "%02X%02X%02X",
+                 otherkey[0] & 0xff, otherkey[1] & 0xff, otherkey[2] & 0xff);
     }
 
     if (path_len >= destsize) {
@@ -85,8 +81,8 @@ static int create_log_path(const Client_Config *c_config, char *dest, int destsi
  * Return 0 on success.
  * Return -1 on failure.
  */
-static int init_logging_session(const Client_Config *c_config, const char *name, const char *selfkey,
-                                const char *otherkey, struct chatlog *log, Log_Type type)
+static int init_logging_session(const Client_Config *c_config, const Paths *paths, const char *name,
+                                const char *selfkey, const char *otherkey, struct chatlog *log, Log_Type type)
 {
     if (log == NULL) {
         return -1;
@@ -96,9 +92,9 @@ static int init_logging_session(const Client_Config *c_config, const char *name,
         return -1;
     }
 
-    char log_path[MAX_STR_SIZE];
+    char log_path[TOXIC_MAX_PATH_LENGTH];
 
-    const int path_len = create_log_path(c_config, log_path, sizeof(log_path), name, selfkey, otherkey);
+    const int path_len = create_log_path(c_config, paths, log_path, sizeof(log_path), name, selfkey, otherkey);
 
     if (path_len == -1 || path_len >= sizeof(log->path)) {
         return -1;
@@ -219,8 +215,8 @@ int log_enable(struct chatlog *log)
  * Return 0 on success.
  * Return -1 on failure.
  */
-int log_init(struct chatlog *log, const Client_Config *c_config, const char *name, const char *selfkey,
-             const char *otherkey, Log_Type type)
+int log_init(struct chatlog *log, const Client_Config *c_config, const Paths *paths, const char *name,
+             const char *selfkey, const char *otherkey, Log_Type type)
 {
     if (log == NULL) {
         return -1;
@@ -231,7 +227,7 @@ int log_init(struct chatlog *log, const Client_Config *c_config, const char *nam
         return -1;
     }
 
-    if (init_logging_session(c_config, name, selfkey, otherkey, log, type) == -1) {
+    if (init_logging_session(c_config, paths, name, selfkey, otherkey, log, type) == -1) {
         return -1;
     }
 
@@ -625,8 +621,8 @@ int load_chat_history(struct chatlog *log, ToxWindow *self, const Client_Config
  * Return 0 on success or if no log exists.
  * Return -1 on failure.
  */
-int rename_logfile(Windows *windows, const Client_Config *c_config, const char *src, const char *dest,
-                   const char *selfkey, const char *otherkey, uint16_t window_id)
+int rename_logfile(Windows *windows, const Client_Config *c_config, const Paths *paths, const char *src,
+                   const char *dest, const char *selfkey, const char *otherkey, uint16_t window_id)
 {
     ToxWindow *toxwin = get_window_pointer_by_id(windows, window_id);
     struct chatlog *log = NULL;
@@ -647,21 +643,21 @@ int rename_logfile(Windows *windows, const Client_Config *c_config, const char *
         log_disable(log);
     }
 
-    char newpath[MAX_STR_SIZE];
-    char oldpath[MAX_STR_SIZE];
+    char newpath[TOXIC_MAX_PATH_LENGTH];
+    char oldpath[TOXIC_MAX_PATH_LENGTH];
 
-    if (create_log_path(c_config, oldpath, sizeof(oldpath), src, selfkey, otherkey) == -1) {
+    if (create_log_path(c_config, paths, oldpath, sizeof(oldpath), src, selfkey, otherkey) == -1) {
         goto on_error;
     }
 
     if (!file_exists(oldpath)) {  // still need to rename path
-        init_logging_session(c_config, dest, selfkey, otherkey, log, LOG_TYPE_CHAT);
+        init_logging_session(c_config, paths, dest, selfkey, otherkey, log, LOG_TYPE_CHAT);
         return 0;
     }
 
-    const int new_path_len = create_log_path(c_config, newpath, sizeof(newpath), dest, selfkey, otherkey);
+    const int new_path_len = create_log_path(c_config, paths, newpath, sizeof(newpath), dest, selfkey, otherkey);
 
-    if (new_path_len == -1 || new_path_len >= MAX_STR_SIZE) {
+    if (new_path_len == -1 || new_path_len >= TOXIC_MAX_PATH_LENGTH) {
         goto on_error;
     }
 
