@@ -1,6 +1,6 @@
 /*  main.c
  *
- *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
+ *  Copyright (C) 2014-2026 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic. Toxic is free software licensed
  *  under the GNU General Public License 3.0.
@@ -48,6 +48,7 @@
 #include "misc_tools.h"
 #include "name_lookup.h"
 #include "notify.h"
+#include "paths.h"
 #include "prompt.h"
 #include "run_options.h"
 #include "settings.h"
@@ -180,7 +181,7 @@ static void load_friendlist(Toxic *toxic)
         friendlist_onFriendAdded(NULL, toxic, i, false);
     }
 
-    sort_friendlist_index();
+    sort_friendlist_index(toxic->friends);
 }
 
 static void load_groups(Toxic *toxic)
@@ -724,8 +725,8 @@ static bool load_toxic(Toxic *toxic, Init_Queue *init_q)
     init_tox_callbacks(toxic->tox);
     load_friendlist(toxic);
 
-    if (load_blocklist(toxic->client_data.block_path) == -1) {
-        init_queue_add(init_q, "Failed to load block list");
+    if (load_blocklist(toxic->client_data.block_path, toxic->blocked) == -1) {
+        init_queue_add(init_q, "Failed to load block list.");
     }
 
     if (tox_self_get_name_size(toxic->tox) == 0) {
@@ -818,7 +819,7 @@ _Noreturn static void *thread_cqueue(void *data)
             if (w->type == WINDOW_TYPE_CHAT) {
                 cqueue_check_unread(w);
 
-                if (get_friend_connection_status(w->num) != TOX_CONNECTION_NONE) {
+                if (get_friend_connection_status(toxic->friends, w->num) != TOX_CONNECTION_NONE) {
                     cqueue_try_send(w, toxic->tox);
                 }
             }
@@ -864,6 +865,9 @@ static void print_usage(void)
     fprintf(stderr, "  -p, --SOCKS5-proxy       Use SOCKS5 proxy: Requires [IP] [port]\n");
     fprintf(stderr, "  -P, --HTTP-proxy         Use HTTP proxy: Requires [IP] [port]\n");
     fprintf(stderr, "  -r, --namelist           Use specified name lookup server list\n");
+#ifdef TOX_EXPERIMENTAL
+    fprintf(stderr, "  -s, --netstats           Dump network stats to log on exit: Requires [path]\n");
+#endif
     fprintf(stderr, "  -t, --force-tcp          Force toxic to use a TCP connection (use with proxies)\n");
     fprintf(stderr, "  -T, --tcp-server         Act as a TCP relay server: Requires [port]\n");
     fprintf(stderr, "  -u, --unencrypt-data     Unencrypt an encrypted data file\n");
@@ -881,6 +885,101 @@ static void set_default_run_options(Run_Options *run_opts)
     run_opts->proxy_type = (uint8_t) TOX_PROXY_TYPE_NONE;
 }
 
+static void handle_opt_config_file(Run_Options *run_opts, Init_Queue *init_q, const char *arg_str)
+{
+    snprintf(run_opts->config_path, sizeof(run_opts->config_path), "%s", arg_str);
+    run_opts->use_custom_config_file = true;
+
+    init_queue_add(init_q, "Using '%s' custom config file", run_opts->config_path);
+}
+
+static void handle_opt_data_file(Toxic *toxic, Run_Options *run_opts, Init_Queue *init_q, const char *arg_str)
+{
+    run_opts->use_custom_data = true;
+
+    Client_Data *client_data = &toxic->client_data;
+
+    free(client_data->data_path);
+    client_data->data_path = NULL;
+
+    free(client_data->block_path);
+    client_data->block_path = NULL;
+
+    client_data->data_path = strdup(arg_str);
+
+    if (client_data->data_path == NULL) {
+        exit_toxic_err(FATALERR_MEMORY, "failed in parse_args");
+    }
+
+    size_t block_path_len = strlen(arg_str) + strlen("-blocklist") + 1;
+    client_data->block_path = malloc(block_path_len);
+
+    if (client_data->block_path == NULL) {
+        exit_toxic_err(FATALERR_MEMORY, "failed in parse_args");
+    }
+
+    snprintf(client_data->block_path, block_path_len, "%s-blocklist", arg_str);
+
+    init_queue_add(init_q, "Using '%s' tox profile", client_data->data_path);
+}
+
+static void handle_opt_logging(Run_Options *run_opts, Init_Queue *init_q, const char *arg_str)
+{
+    run_opts->logging = true;
+
+    if (strcmp(arg_str, "stderr") != 0) {
+        run_opts->log_fp = fopen(arg_str, "w");
+
+        if (run_opts->log_fp != NULL) {
+            init_queue_add(init_q, "Toxcore logging enabled to %s", arg_str);
+        } else {
+            run_opts->debug = true;
+            init_queue_add(init_q, "Failed to open log file %s. Falling back to stderr.", arg_str);
+        }
+    } else {
+        run_opts->debug = true;
+        init_queue_add(init_q, "Toxcore logging enabled to stderr");
+    }
+}
+
+static void handle_opt_proxy(Run_Options *run_opts, Init_Queue *init_q, const char *arg_str, int *opt_idx, int argc,
+                             char *argv[])
+{
+    if (run_opts->proxy_type == TOX_PROXY_TYPE_NONE) {
+        run_opts->proxy_type = TOX_PROXY_TYPE_HTTP;
+    }
+
+    snprintf(run_opts->proxy_address, sizeof(run_opts->proxy_address), "%s", arg_str);
+
+    if (++(*opt_idx) > argc || argv[*opt_idx - 1][0] == '-') {
+        exit_toxic_err(FATALERR_PROXY, "Proxy error");
+    }
+
+    const long int port = strtol(argv[*opt_idx - 1], NULL, 10);
+
+    if (port <= 0 || port > MAX_PORT_RANGE) {
+        exit_toxic_err(FATALERR_PROXY, "Proxy error");
+    }
+
+    run_opts->proxy_port = port;
+}
+
+#ifdef TOX_EXPERIMENTAL
+static void handle_opt_netstats(Run_Options *run_opts, Init_Queue *init_q, const char *arg_str)
+{
+    run_opts->netprof_fp = fopen(arg_str, "w");
+
+    if (run_opts->netprof_fp != NULL) {
+        init_queue_add(init_q, "Network profile logging enabled. Logging to file: '%s'", arg_str);
+        run_opts->netprof_log_dump = true;
+        run_opts->netprof_start_time = time(NULL);
+    } else {
+        init_queue_add(init_q, "Failed to open file '%s' for network profile logging.", arg_str);
+    }
+}
+
+#endif
+
 static void parse_args(Toxic *toxic, Init_Queue *init_q, int argc, char *argv[])
 {
     Run_Options *run_opts = toxic->run_opts;
@@ -898,6 +997,9 @@ static void parse_args(Toxic *toxic, Init_Queue *init_q, int argc, char *argv[])
         {"help", no_argument, 0, 'h'},
         {"noconnect", no_argument, 0, 'o'},
         {"namelist", required_argument, 0, 'r'},
+#ifdef TOX_EXPERIMENTAL
+        {"netstats", required_argument, 0, 's'},
+#endif
         {"force-tcp", no_argument, 0, 't'},
         {"tcp-server", required_argument, 0, 'T'},
         {"SOCKS5-proxy", required_argument, 0, 'p'},
@@ -907,7 +1009,12 @@ static void parse_args(Toxic *toxic, Init_Queue *init_q, int argc, char *argv[])
         {NULL, no_argument, NULL, 0},
     };
 
+#ifdef TOX_EXPERIMENTAL
+    const char *opts_str = "4bdehLotuxvc:f:l:n:r:s:p:P:T:";
+#else
     const char *opts_str = "4bdehLotuxvc:f:l:n:r:p:P:T:";
+#endif // TOX_EXPERIMENTAL
+
     int opt = 0;
     int indexptr = 0;
 
@@ -925,15 +1032,7 @@ static void parse_args(Toxic *toxic, Init_Queue *init_q, int argc, char *argv[])
             }
 
             case 'c': {
-                if (optarg == NULL) {
-                    init_queue_add(init_q, "Invalid argument for option: %d", opt);
-                    break;
-                }
-
-                snprintf(run_opts->config_path, sizeof(run_opts->config_path), "%s", optarg);
-                run_opts->use_custom_config_file = true;
-
-                init_queue_add(init_q, "Using '%s' custom config file", run_opts->config_path);
+                handle_opt_config_file(run_opts, init_q, optarg);
                 break;
             }
 
@@ -949,64 +1048,13 @@ static void parse_args(Toxic *toxic, Init_Queue *init_q, int argc, char *argv[])
             }
 
             case 'f': {
-                if (optarg == NULL) {
-                    init_queue_add(init_q, "Invalid argument for option: %d", opt);
-                    break;
-                }
-
-                run_opts->use_custom_data = true;
-
-                Client_Data *client_data = &toxic->client_data;
-
-                if (client_data->data_path) {
-                    free(client_data->data_path);
-                    client_data->data_path = NULL;
-                }
-
-                if (client_data->block_path) {
-                    free(client_data->block_path);
-                    client_data->block_path = NULL;
-                }
-
-                client_data->data_path = malloc(strlen(optarg) + 1);
-
-                if (client_data->data_path == NULL) {
-                    exit_toxic_err(FATALERR_MEMORY, "failed in parse_args");
-                }
-
-                strcpy(client_data->data_path, optarg);
-
-                client_data->block_path = malloc(strlen(optarg) + strlen("-blocklist") + 1);
-
-                if (client_data->block_path == NULL) {
-                    exit_toxic_err(FATALERR_MEMORY, "failed in parse_args");
-                }
-
-                strcpy(client_data->block_path, optarg);
-                strcat(client_data->block_path, "-blocklist");
-
-                init_queue_add(init_q, "Using '%s' tox profile", client_data->data_path);
-
+                handle_opt_data_file(toxic, run_opts, init_q, optarg);
                 break;
             }
 
             case 'l': {
                 if (optarg) {
-                    run_opts->logging = true;
-
-                    if (strcmp(optarg, "stderr") != 0) {
-                        run_opts->log_fp = fopen(optarg, "w");
-
-                        if (run_opts->log_fp != NULL) {
-                            init_queue_add(init_q, "Toxcore logging enabled to %s", optarg);
-                        } else {
-                            run_opts->debug = true;
-                            init_queue_add(init_q, "Failed to open log file %s. Falling back to stderr.", optarg);
-                        }
-                    } else {
-                        run_opts->debug = true;
-                        init_queue_add(init_q, "Toxcore logging enabled to stderr");
-                    }
+                    handle_opt_logging(run_opts, init_q, optarg);
                 }
 
                 break;
@@ -1047,23 +1095,7 @@ static void parse_args(Toxic *toxic, Init_Queue *init_q, int argc, char *argv[])
                     break;
                 }
 
-                if (run_opts->proxy_type == TOX_PROXY_TYPE_NONE) {
-                    run_opts->proxy_type = TOX_PROXY_TYPE_HTTP;
-                }
-
-                snprintf(run_opts->proxy_address, sizeof(run_opts->proxy_address), "%s", optarg);
-
-                if (++optind > argc || argv[optind - 1][0] == '-') {
-                    exit_toxic_err(FATALERR_PROXY, "Proxy error");
-                }
-
-                const long int port = strtol(argv[optind - 1], NULL, 10);
-
-                if (port <= 0 || port > MAX_PORT_RANGE) {
-                    exit_toxic_err(FATALERR_PROXY, "Proxy error");
-                }
-
-                run_opts->proxy_port = port;
+                handle_opt_proxy(run_opts, init_q, optarg, &optind, argc, argv);
                 break;
             }
 
@@ -1103,6 +1135,20 @@ static void parse_args(Toxic *toxic, Init_Queue *init_q, int argc, char *argv[])
                 break;
             }
 
+#ifdef TOX_EXPERIMENTAL
+
+            case 's': {
+                if (optarg == NULL) {
+                    init_queue_add(init_q, "Invalid argument for option: %d", opt);
+                    break;
+                }
+
+                handle_opt_netstats(run_opts, init_q, optarg);
+                break;
+            }
+
+#endif // TOX_EXPERIMENTAL
+
             case 'u': {
                 run_opts->unencrypt_data = true;
                 break;
@@ -1128,9 +1174,9 @@ static void parse_args(Toxic *toxic, Init_Queue *init_q, int argc, char *argv[])
  *
  * Exits the process with an error on failure.
  */
-static void init_default_data_files(Client_Data *client_data)
+static void init_default_data_files(Client_Data *client_data, const Paths *paths)
 {
-    char *user_config_dir = get_user_config_dir();
+    char *user_config_dir = get_user_config_dir(paths);
 
     if (user_config_dir == NULL) {
         exit_toxic_err(FATALERR_FILEOP, "failed in init_default_data_files()");
@@ -1146,20 +1192,18 @@ static void init_default_data_files(Client_Data *client_data)
             exit_toxic_err(FATALERR_MEMORY, "strdup() failed in init_default_data_files()");
         }
     } else {
-        client_data->data_path = malloc(strlen(user_config_dir) + strlen(CONFIGDIR) + strlen(DATANAME) + 1);
-        client_data->block_path = malloc(strlen(user_config_dir) + strlen(CONFIGDIR) + strlen(BLOCKNAME) + 1);
+        size_t data_path_len = strlen(user_config_dir) + strlen(CONFIGDIR) + strlen(DATANAME) + 1;
+        client_data->data_path = malloc(data_path_len);
+
+        size_t block_path_len = strlen(user_config_dir) + strlen(CONFIGDIR) + strlen(BLOCKNAME) + 1;
+        client_data->block_path = malloc(block_path_len);
 
         if (client_data->data_path == NULL || client_data->block_path == NULL) {
             exit_toxic_err(FATALERR_MEMORY, "malloc() failed in init_default_data_files()");
         }
 
-        strcpy(client_data->data_path, user_config_dir);
-        strcat(client_data->data_path, CONFIGDIR);
-        strcat(client_data->data_path, DATANAME);
-
-        strcpy(client_data->block_path, user_config_dir);
-        strcat(client_data->block_path, CONFIGDIR);
-        strcat(client_data->block_path, BLOCKNAME);
+        snprintf(client_data->data_path, data_path_len, "%s%s%s", user_config_dir, CONFIGDIR, DATANAME);
+        snprintf(client_data->block_path, block_path_len, "%s%s%s", user_config_dir, CONFIGDIR, BLOCKNAME);
     }
 
     free(user_config_dir);
@@ -1205,6 +1249,38 @@ static Toxic *toxic_init(void)
 
     toxic->windows = tmp_windows;
 
+#ifdef AUDIO
+    toxic->call_control = calloc(1, sizeof(struct CallControl));
+
+    if (toxic->call_control == NULL) {
+        free(toxic->c_config);
+        free(toxic->run_opts);
+        free(toxic->windows);
+        free(toxic->friends);
+        free(toxic->blocked);
+        free(toxic);
+        return NULL;
+    }
+
+#endif
+
+    init_friendlist(toxic);
+
+    toxic->paths = paths_init();
+
+    if (toxic->paths == NULL) {
+        free(toxic->c_config);
+        free(toxic->run_opts);
+        free(toxic->windows);
+        free(toxic->friends);
+        free(toxic->blocked);
+#ifdef AUDIO
+        free(toxic->call_control);
+#endif
+        free(toxic);
+        return NULL;
+    }
+
     return toxic;
 }
 
@@ -1248,7 +1324,7 @@ int main(int argc, char **argv)
     }
 
     if (!run_opts->use_custom_data) {
-        init_default_data_files(&toxic->client_data);
+        init_default_data_files(&toxic->client_data, toxic->paths);
     }
 
     const bool datafile_exists = file_exists(toxic->client_data.data_path);
@@ -1260,7 +1336,7 @@ int main(int argc, char **argv)
         first_time_encrypt(&toxic->client_data, init_q, "Encrypt existing data file? Y/n (q to quit)");
     }
 
-    if (!settings_load_config_file(run_opts, toxic->client_data.data_path)) {
+    if (!settings_load_config_file(run_opts, toxic->paths, toxic->client_data.data_path)) {
         init_queue_add(init_q, "Failed to load config file");
     }
 
@@ -1311,7 +1387,7 @@ int main(int argc, char **argv)
     load_groups(toxic);
     load_conferences(toxic);
 
-    const int fs_ret = settings_load_friends(run_opts);
+    const int fs_ret = settings_load_friends(toxic->friends, run_opts);
 
     if (fs_ret != 0) {
         init_queue_add(init_q, "Failed to load friend config settings: error %d", fs_ret);
@@ -1391,7 +1467,7 @@ int main(int argc, char **argv)
 
 #endif /* PYTHON */
 
-    init_notify(60, c_config->notification_timeout);
+    init_notify(60, c_config->notification_timeout, c_config->device_cooldown);
 
     /* screen/tmux auto-away timer */
     if (init_mplex_away_timer(toxic) == -1) {
@@ -1413,7 +1489,7 @@ int main(int argc, char **argv)
     init_q = NULL;
 
     /* set user avatar from config file. if no path is supplied tox_unset_avatar is called */
-    char avatarstr[PATH_MAX + 11];
+    char avatarstr[TOXIC_MAX_PATH_LENGTH + 11];
     snprintf(avatarstr, sizeof(avatarstr), "/avatar %s", c_config->avatar_path);
     execute(home_window->chatwin->history, home_window, toxic, avatarstr, GLOBAL_COMMAND_MODE);
 
