@@ -1,6 +1,6 @@
 /*  prompt.c
  *
- *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
+ *  Copyright (C) 2014-2026 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic. Toxic is free software licensed
  *  under the GNU General Public License 3.0.
@@ -22,6 +22,7 @@
 #include "line_info.h"
 #include "log.h"
 #include "misc_tools.h"
+#include "netprof.h"
 #include "notify.h"
 #include "prompt.h"
 #include "settings.h"
@@ -31,9 +32,6 @@
 
 extern struct Winthread Winthread;
 
-extern FriendsList Friends;
-FriendRequests FrndRequests;
-
 /* Array of global command names used for tab completion. */
 static const char *const glob_cmd_list[] = {
     "/accept",
@@ -167,23 +165,23 @@ Tox_Connection prompt_selfConnectionStatus(Toxic *toxic)
 
 /* Adds friend request to pending friend requests.
    Returns request number on success, -1 if queue is full. */
-static int add_friend_request(const char *public_key, const char *data)
+static int add_friend_request(Toxic *toxic, const char *public_key, const char *data)
 {
-    if (FrndRequests.max_idx >= MAX_FRIEND_REQUESTS) {
+    if (toxic->frnd_requests.max_idx >= MAX_FRIEND_REQUESTS) {
         return -1;
     }
 
-    for (int i = 0; i <= FrndRequests.max_idx; ++i) {
-        if (!FrndRequests.request[i].active) {
-            FrndRequests.request[i].active = true;
-            memcpy(FrndRequests.request[i].key, public_key, TOX_PUBLIC_KEY_SIZE);
-            snprintf(FrndRequests.request[i].msg, sizeof(FrndRequests.request[i].msg), "%s", data);
+    for (int i = 0; i <= toxic->frnd_requests.max_idx; ++i) {
+        if (!toxic->frnd_requests.request[i].active) {
+            toxic->frnd_requests.request[i].active = true;
+            memcpy(toxic->frnd_requests.request[i].key, public_key, TOX_PUBLIC_KEY_SIZE);
+            snprintf(toxic->frnd_requests.request[i].msg, sizeof(toxic->frnd_requests.request[i].msg), "%s", data);
 
-            if (i == FrndRequests.max_idx) {
-                ++FrndRequests.max_idx;
+            if (i == toxic->frnd_requests.max_idx) {
+                ++toxic->frnd_requests.max_idx;
             }
 
-            ++FrndRequests.num_requests;
+            ++toxic->frnd_requests.num_requests;
 
             return i;
         }
@@ -309,6 +307,80 @@ static bool prompt_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr)
     return input_ret;
 }
 
+#ifdef TOX_EXPERIMENTAL
+#define NET_INFO_REFRESH_INTERVAL 1
+static void draw_network_info(const Tox *tox, StatusBar *stb)
+{
+    WINDOW *win = stb->topline;
+
+    wattron(win, COLOR_PAIR(BAR_TEXT));
+    wprintw(win, "%s", stb->network_info);
+    wattroff(win, COLOR_PAIR(BAR_TEXT));
+
+    if (!timed_out(stb->time_last_refreshed, NET_INFO_REFRESH_INTERVAL)) {
+        return;
+    }
+
+    stb->time_last_refreshed = get_unix_time();
+
+    const uint64_t up_bytes = netprof_get_bytes_up(tox);
+    const uint64_t down_bytes = netprof_get_bytes_down(tox);
+
+    const uint64_t up_delta = (up_bytes - stb->up_bytes) / NET_INFO_REFRESH_INTERVAL;
+    const uint64_t down_delta = (down_bytes - stb->down_bytes) / NET_INFO_REFRESH_INTERVAL;
+
+    stb->up_bytes = up_bytes;
+    stb->down_bytes = down_bytes;
+
+    double up = (double)up_bytes;
+    double down = (double)down_bytes;
+    const char *up_unit = "bytes";
+    const char *down_unit = "bytes";
+
+    if (up_bytes > MiB) {
+        up /= (double)MiB;
+        up_unit = "MiB";
+    } else if (up_bytes > KiB) {
+        up /= (double)KiB;
+        up_unit = "KiB";
+    }
+
+    if (down_bytes > MiB) {
+        down /= (double)MiB;
+        down_unit = "MiB";
+    } else if (down_bytes > KiB) {
+        down /= (double)KiB;
+        down_unit = "KiB";
+    }
+
+    double up_bps = (double)up_delta;
+    double down_bps = (double)down_delta;
+    const char *up_bps_unit = "b/s";
+    const char *down_bps_unit = "b/s";
+
+    if (up_bps > MiB) {
+        up_bps /= (double)MiB;
+        up_bps_unit = "MiB/s";
+    } else if (up_bps > KiB) {
+        up_bps /= (double)KiB;
+        up_bps_unit = "KiB/s";
+    }
+
+    if (down_bps > MiB) {
+        down_bps /= (double)MiB;
+        down_bps_unit = "MiB/s";
+    } else if (down_bps > KiB) {
+        down_bps /= (double)KiB;
+        down_bps_unit = "KiB/s";
+    }
+
+    snprintf(stb->network_info, sizeof(stb->network_info),
+             " | [Up: %.1f%s (%.1f%s) | Down: %.1f%s (%.1f%s)]", up, up_unit, up_bps, up_bps_unit,
+             down, down_unit, down_bps, down_bps_unit);
+}
+
+#endif // TOX_EXPERIMENTAL
+
 static void prompt_onDraw(ToxWindow *self, Toxic *toxic)
 {
     if (toxic == NULL || self == NULL) {
@@ -471,6 +543,14 @@ static void prompt_onDraw(ToxWindow *self, Toxic *toxic)
         wattroff(statusbar->topline, COLOR_PAIR(BAR_TEXT));
     }
 
+#ifdef TOX_EXPERIMENTAL
+
+    if (toxic->c_config->show_network_info) {
+        draw_network_info(toxic->tox, statusbar);
+    }
+
+#endif // TOX_EXPERIMENTAL
+
     int y;
     int x;
     getyx(self->window, y, x);
@@ -499,16 +579,17 @@ static void prompt_onConnectionChange(ToxWindow *self, Toxic *toxic, uint32_t fr
     const Client_Config *c_config = toxic->c_config;
     ChatContext *ctx = self->chatwin;
 
-    if (!friend_config_get_show_connection_msg(friendnum)) {
+    if (!friend_config_get_show_connection_msg(toxic->friends, friendnum)) {
         return;
     }
 
     char nick[TOXIC_MAX_NAME_LENGTH + 1];
-    get_friend_name(nick, sizeof(nick), friendnum);
+    get_friend_name(toxic->friends, nick, sizeof(nick), friendnum);
 
     const char *msg;
 
-    if (connection_status != TOX_CONNECTION_NONE && Friends.list[friendnum].connection_status == TOX_CONNECTION_NONE) {
+    if (connection_status != TOX_CONNECTION_NONE
+            && toxic->friends->list[friendnum].connection_status == TOX_CONNECTION_NONE) {
         msg = "has come online";
         line_info_add(self, c_config, true, nick, NULL, CONNECTION, 0, GREEN, "%s", msg);
         write_to_log(ctx->log, c_config, msg, nick, LOG_HINT_CONNECT);
@@ -538,10 +619,14 @@ static void prompt_onConnectionChange(ToxWindow *self, Toxic *toxic, uint32_t fr
 /**
  * Return true is the first 3 bytes of `key` are identical to any other contact in the contact list.
  */
-static bool key_is_similar(const char *key)
+static bool key_is_similar(const FriendsList *friends, const char *key)
 {
-    for (size_t i = 0; i < Friends.max_idx; ++i) {
-        const ToxicFriend *friend = &Friends.list[i];
+    if (friends == NULL) {
+        return false;
+    }
+
+    for (size_t i = 0; i < friends->max_idx; ++i) {
+        const ToxicFriend *friend = &friends->list[i];
 
         if (!friend->active) {
             continue;
@@ -567,14 +652,14 @@ static void prompt_onFriendRequest(ToxWindow *self, Toxic *toxic, const char *ke
 
     line_info_add(self, c_config, true, NULL, NULL, SYS_MSG, 0, 0, "Friend request with the message '%s'", data);
 
-    if (key_is_similar(key)) {
+    if (key_is_similar(toxic->friends, key)) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED,
                       "WARNING: This contact's public key is suspiciously similar to that of another contact ");
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED,
                       "in your list. This may be an impersonation attempt, or it may have occurred by chance.");
     }
 
-    const int n = add_friend_request(key, data);
+    const int n = add_friend_request(toxic, key, data);
 
     if (n == -1) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Friend request queue is full. Discarding request.");
@@ -660,7 +745,7 @@ static void prompt_init_log(Toxic *toxic)
     char myid[TOX_ADDRESS_SIZE];
     tox_self_get_address(toxic->tox, (uint8_t *) myid);
 
-    if (log_init(ctx->log, toxic->c_config, self->name, myid, NULL, LOG_TYPE_PROMPT) != 0) {
+    if (log_init(ctx->log, toxic->c_config, toxic->paths, self->name, myid, NULL, LOG_TYPE_PROMPT) != 0) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Warning: Log failed to initialize.");
         return;
     }
