@@ -1,6 +1,6 @@
 /*  prompt.h
  *
- *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
+ *  Copyright (C) 2014-2026 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic. Toxic is free software licensed
  *  under the GNU General Public License 3.0.
@@ -12,22 +12,6 @@
 #include "toxic.h"
 #include "windows.h"
 
-#define MAX_FRIEND_REQUESTS 20
-
-struct friend_request {
-    bool active;
-    char msg[TOX_MAX_FRIEND_REQUEST_LENGTH + 1];
-    uint8_t key[TOX_PUBLIC_KEY_SIZE];
-};
-
-typedef struct FriendRequests {
-    int max_idx;
-    int num_requests;
-    struct friend_request request[MAX_FRIEND_REQUESTS];
-} FriendRequests;
-
-extern FriendRequests FrndRequests;
-
 ToxWindow *new_prompt(void);
 
 void prep_prompt_win(void);
