@@ -1,7 +1,7 @@
 /*  run_options.h
  *
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
+ *  Copyright (C) 2026 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -26,6 +26,7 @@
 #include <stdbool.h>
 #include <stdio.h>  // needed for FILE
 #include <stdint.h>
+#include <time.h>
 
 #include "toxic_constants.h"
 
@@ -41,13 +42,17 @@ typedef struct Run_Options {
     bool encrypt_data;
     bool unencrypt_data;
 
-    char nameserver_path[MAX_STR_SIZE];
-    char config_path[MAX_STR_SIZE];
-    char nodes_path[MAX_STR_SIZE];
+    char nameserver_path[TOXIC_MAX_PATH_LENGTH];
+    char config_path[TOXIC_MAX_PATH_LENGTH];
+    char nodes_path[TOXIC_MAX_PATH_LENGTH];
 
     bool logging;
     FILE *log_fp;
 
+    bool netprof_log_dump;
+    FILE *netprof_fp;
+    time_t netprof_start_time;
+
     char proxy_address[256];
     uint8_t proxy_type;
     uint16_t proxy_port;
