@@ -1,6 +1,6 @@
 /*  settings.c
  *
- *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
+ *  Copyright (C) 2014-2026 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic. Toxic is free software licensed
  *  under the GNU General Public License 3.0.
@@ -17,6 +17,7 @@
 #include "friendlist.h"
 #include "groupchats.h"
 #include "misc_tools.h"
+#include "term_mplex.h"
 #include "notify.h"
 #include "toxic.h"
 #include "windows.h"
@@ -35,7 +36,8 @@
 #define TOXIC_CONF_FILE_EXT ".conf"
 #define TOXIC_CONF_FILE_EXT_LENGTH (sizeof(TOXIC_CONF_FILE_EXT) - 1)
 
-static_assert(MAX_STR_SIZE > TOXIC_CONF_FILE_EXT_LENGTH, "MAX_STR_SIZE <= TOXIC_CONF_FILE_EXT_LENGTH");
+static_assert(TOXIC_MAX_PATH_LENGTH > TOXIC_CONF_FILE_EXT_LENGTH,
+              "TOXIC_MAX_PATH_LENGTH <= TOXIC_CONF_FILE_EXT_LENGTH");
 
 #ifdef SOUND_NOTIFY
 #define NO_SOUND "silent"
@@ -62,8 +64,10 @@ static struct ui_strings {
     const char *show_welcome_msg;
     const char *show_connection_msg;
     const char *show_group_connection_msg;
+    const char *show_network_info;
     const char *nodeslist_update_freq;
     const char *autosave_freq;
+    const char *device_cooldown;
 
     const char *line_padding;
     const char *line_join;
@@ -102,8 +106,10 @@ static struct ui_strings {
     "show_welcome_msg",
     "show_connection_msg",
     "show_group_connection_msg",
+    "show_network_info",
     "nodeslist_update_freq",
     "autosave_freq",
+    "device_cooldown",
     "line_padding",
     "line_join",
     "line_quit",
@@ -141,15 +147,17 @@ static void ui_defaults(Client_Config *settings)
     settings->show_welcome_msg = true;
     settings->show_connection_msg = true;
     settings->show_group_connection_msg = true;
+    settings->show_network_info = false;
     settings->nodeslist_update_freq = 1;
     settings->autosave_freq = 600;
+    settings->device_cooldown = 5;
 
     settings->line_padding = true;
-    snprintf(settings->line_join, LINE_HINT_MAX + 1, "%s", LINE_JOIN);
-    snprintf(settings->line_quit, LINE_HINT_MAX + 1, "%s", LINE_QUIT);
-    snprintf(settings->line_alert, LINE_HINT_MAX + 1, "%s", LINE_ALERT);
-    snprintf(settings->line_normal, LINE_HINT_MAX + 1, "%s", LINE_NORMAL);
-    snprintf(settings->line_special, LINE_HINT_MAX + 1, "%s", LINE_SPECIAL);
+    snprintf(settings->line_join, sizeof(settings->line_join), "%s", LINE_JOIN);
+    snprintf(settings->line_quit, sizeof(settings->line_quit), "%s", LINE_QUIT);
+    snprintf(settings->line_alert, sizeof(settings->line_alert), "%s", LINE_ALERT);
+    snprintf(settings->line_normal, sizeof(settings->line_normal), "%s", LINE_NORMAL);
+    snprintf(settings->line_special, sizeof(settings->line_special), "%s", LINE_SPECIAL);
 
     settings->mplex_away = true;
     snprintf(settings->mplex_away_note, sizeof(settings->mplex_away_note), "%s", MPLEX_AWAY_NOTE);
@@ -216,11 +224,11 @@ static const struct tox_strings {
 
 static void tox_defaults(Client_Config *settings)
 {
-    strcpy(settings->download_path, "");
-    strcpy(settings->chatlogs_path, "");
-    strcpy(settings->avatar_path, "");
-    strcpy(settings->autorun_path, "");
-    strcpy(settings->password_eval, "");
+    settings->download_path[0] = '\0';
+    settings->chatlogs_path[0] = '\0';
+    settings->avatar_path[0] = '\0';
+    settings->autorun_path[0] = '\0';
+    settings->password_eval[0] = '\0';
 }
 
 #ifdef AUDIO
@@ -360,14 +368,14 @@ static void set_key_binding(int *key, const char **bind)
     }
 }
 
-bool settings_load_config_file(Run_Options *run_opts, const char *data_path)
+bool settings_load_config_file(Run_Options *run_opts, const Paths *paths, const char *data_path)
 {
-    char tmp_path[MAX_STR_SIZE] = {0};
+    char tmp_path[TOXIC_MAX_PATH_LENGTH] = {0};
 
     if (run_opts->use_custom_config_file) {
         snprintf(tmp_path, sizeof(tmp_path), "%s", run_opts->config_path);
     } else if (run_opts->use_custom_data) {
-        char tmp_data[MAX_STR_SIZE - TOXIC_CONF_FILE_EXT_LENGTH];
+        char tmp_data[TOXIC_MAX_PATH_LENGTH - TOXIC_CONF_FILE_EXT_LENGTH];
 
         if (strlen(data_path) >= sizeof(tmp_data)) {
             return false;
@@ -383,7 +391,7 @@ bool settings_load_config_file(Run_Options *run_opts, const char *data_path)
 
         snprintf(tmp_path, sizeof(tmp_path), "%s%s", tmp_data, TOXIC_CONF_FILE_EXT);
     } else {
-        char *user_config_dir = get_user_config_dir();
+        char *user_config_dir = get_user_config_dir(paths);
         snprintf(tmp_path, sizeof(tmp_path), "%s%stoxic%s", user_config_dir, CONFIGDIR, TOXIC_CONF_FILE_EXT);
         free(user_config_dir);
     }
@@ -566,7 +574,7 @@ int settings_load_groups(Windows *windows, const Run_Options *run_opts)
     return 0;
 }
 
-int settings_load_friends(const Run_Options *run_opts)
+int settings_load_friends(FriendsList *friends, const Run_Options *run_opts)
 {
     config_t cfg[1];
     config_init(cfg);
@@ -599,7 +607,7 @@ int settings_load_friends(const Run_Options *run_opts)
         }
 
         if (config_setting_lookup_string(keys, friend_strings.tab_name_color, &str)) {
-            if (!friend_config_set_tab_name_colour(public_key, str)) {
+            if (!friend_config_set_tab_name_colour(friends, public_key, str)) {
                 fprintf(stderr, "config error: failed to set friend tab name color for %s: (color: %s)\n",
                         public_key, str);
             }
@@ -608,7 +616,7 @@ int settings_load_friends(const Run_Options *run_opts)
         int autolog_enabled;
 
         if (config_setting_lookup_bool(keys, friend_strings.autolog, &autolog_enabled)) {
-            if (!friend_config_set_autolog(public_key, autolog_enabled != 0)) {
+            if (!friend_config_set_autolog(friends, public_key, autolog_enabled != 0)) {
                 fprintf(stderr, "config error: failed to apply friend autolog setting for: %s\n", public_key);
             }
         }
@@ -616,7 +624,7 @@ int settings_load_friends(const Run_Options *run_opts)
         int auto_accept_files;
 
         if (config_setting_lookup_bool(keys, friend_strings.auto_accept_files, &auto_accept_files)) {
-            if (!friend_config_set_auto_accept_files(public_key, auto_accept_files != 0)) {
+            if (!friend_config_set_auto_accept_files(friends, public_key, auto_accept_files != 0)) {
                 fprintf(stderr,
                         "config error: failed to apply friend auto-accept filetransfers setting for: %s\n",
                         public_key);
@@ -626,7 +634,7 @@ int settings_load_friends(const Run_Options *run_opts)
         int show_connection_msg;
 
         if (config_setting_lookup_bool(keys, friend_strings.show_connection_msg, &show_connection_msg)) {
-            if (!friend_config_set_show_connection_msg(public_key, show_connection_msg != 0)) {
+            if (!friend_config_set_show_connection_msg(friends, public_key, show_connection_msg != 0)) {
                 fprintf(stderr,
                         "config error: failed to apply friend show connection message setting for: %s\n",
                         public_key);
@@ -634,7 +642,7 @@ int settings_load_friends(const Run_Options *run_opts)
         }
 
         if (config_setting_lookup_string(keys, friend_strings.alias, &str)) {
-            if (!friend_config_set_alias(public_key, str, strlen(str))) {
+            if (!friend_config_set_alias(friends, public_key, str, strlen(str))) {
                 fprintf(stderr, "config error: failed to apply alias '%s' for: %s\n", str, public_key);
             }
         }
@@ -667,6 +675,7 @@ int settings_load_blocked_words(Client_Data *client_data, const Run_Options *run
     const int list_size = config_setting_length(setting);
 
     if (list_size <= 0) {
+        config_destroy(cfg);
         return 0;
     }
 
@@ -675,6 +684,7 @@ int settings_load_blocked_words(Client_Data *client_data, const Run_Options *run
 
     if (words_list == NULL) {
         fprintf(stderr, "config error: failed to allocate memory for blocked words list.\n");
+        config_destroy(cfg);
         return -3;
     }
 
@@ -703,299 +713,310 @@ int settings_load_blocked_words(Client_Data *client_data, const Run_Options *run
     return 0;
 }
 
-int settings_load_main(Client_Config *s, const Run_Options *run_opts)
+static void settings_load_ui(config_t *cfg, Client_Config *s)
 {
-    config_t cfg[1];
-    config_init(cfg);
-
-    config_setting_t *setting = NULL;
-
-    /* Load default settings */
-    ui_defaults(s);
-    tox_defaults(s);
-    key_defaults(s);
-
-#ifdef AUDIO
-    audio_defaults(s);
-#endif
-
-    const int c_ret = settings_init_config(cfg, run_opts);
-
-    if (c_ret < 0) {
-        config_destroy(cfg);
-        return c_ret;
-    }
-
+    config_setting_t *setting = config_lookup(cfg, ui_strings.self);
     const char *str = NULL;
     int bool_val;
 
-    /* ui */
-    if ((setting = config_lookup(cfg, ui_strings.self)) != NULL) {
-        if (config_setting_lookup_bool(setting, ui_strings.timestamps, &bool_val)) {
-            s->show_timestamps = bool_val != 0;
-        }
+    if (setting == NULL) {
+        return;
+    }
 
-        int time = 24;
+    if (config_setting_lookup_bool(setting, ui_strings.timestamps, &bool_val)) {
+        s->show_timestamps = bool_val != 0;
+    }
 
-        if (config_setting_lookup_int(setting, ui_strings.time_format, &time)) {
-            if (time == 12) {
-                snprintf(s->timestamp_format, sizeof(s->timestamp_format), "%s", "%I:%M %p");
-                snprintf(s->log_timestamp_format, sizeof(s->log_timestamp_format), "%s", "%Y/%m/%d [%I:%M %p]");
-            }
-        }
+    int time = 24;
 
-        if (config_setting_lookup_string(setting, ui_strings.timestamp_format, &str)) {
-            snprintf(s->timestamp_format, sizeof(s->timestamp_format), "%s", str);
-        }
-
-        if (config_setting_lookup_string(setting, ui_strings.color_bar_bg, &str)) {
-            snprintf(s->color_bar_bg, sizeof(s->color_bar_bg), "%s", str);
-        }
-
-        if (config_setting_lookup_string(setting, ui_strings.color_bar_fg, &str)) {
-            snprintf(s->color_bar_fg, sizeof(s->color_bar_fg), "%s", str);
-        }
-
-        if (config_setting_lookup_string(setting, ui_strings.color_bar_accent, &str)) {
-            snprintf(s->color_bar_accent, sizeof(s->color_bar_accent), "%s", str);
-        }
-
-        if (config_setting_lookup_string(setting, ui_strings.color_bar_notify, &str)) {
-            snprintf(s->color_bar_notify, sizeof(s->color_bar_notify), "%s", str);
-        }
-
-        if (config_setting_lookup_string(setting, ui_strings.log_timestamp_format, &str)) {
-            snprintf(s->log_timestamp_format, sizeof(s->log_timestamp_format), "%s", str);
-        }
-
-        if (config_setting_lookup_bool(setting, ui_strings.alerts, &bool_val)) {
-            s->alerts = bool_val != 0;
-        }
-
-        if (config_setting_lookup_bool(setting, ui_strings.show_notification_content, &bool_val)) {
-            s->show_notification_content = bool_val != 0;
-        }
-
-        if (config_setting_lookup_bool(setting, ui_strings.bell_on_message, &bool_val)) {
-            s->bell_on_message = bool_val != 0 ? NT_BEEP : 0;
-        }
-
-        if (config_setting_lookup_bool(setting, ui_strings.bell_on_filetrans, &bool_val)) {
-            s->bell_on_filetrans = bool_val != 0 ? NT_BEEP : 0;
-        }
-
-        if (config_setting_lookup_bool(setting, ui_strings.bell_on_filetrans_accept, &bool_val)) {
-            s->bell_on_filetrans_accept = bool_val != 0 ? NT_BEEP : 0;
-        }
-
-        if (config_setting_lookup_bool(setting, ui_strings.bell_on_invite, &bool_val)) {
-            s->bell_on_invite = bool_val != 0 ? NT_BEEP : 0;
-        }
-
-        if (config_setting_lookup_bool(setting, ui_strings.autolog, &bool_val)) {
-            s->autolog = bool_val != 0;
-        }
-
-        if (config_setting_lookup_bool(setting, ui_strings.native_colors, &bool_val)) {
-            s->native_colors = bool_val != 0;
-        }
-
-        if (config_setting_lookup_bool(setting, ui_strings.show_typing_self, &bool_val)) {
-            s->show_typing_self = bool_val != 0;
-        }
-
-        if (config_setting_lookup_bool(setting, ui_strings.show_typing_other, &bool_val)) {
-            s->show_typing_other = bool_val != 0;
-        }
-
-        if (config_setting_lookup_bool(setting, ui_strings.show_welcome_msg, &bool_val)) {
-            s->show_welcome_msg = bool_val != 0;
-        }
-
-        if (config_setting_lookup_bool(setting, ui_strings.show_connection_msg, &bool_val)) {
-            s->show_connection_msg = bool_val != 0;
-        }
-
-        if (config_setting_lookup_bool(setting, ui_strings.show_group_connection_msg, &bool_val)) {
-            s->show_group_connection_msg = bool_val != 0;
-        }
-
-        config_setting_lookup_int(setting, ui_strings.history_size, &s->history_size);
-        config_setting_lookup_int(setting, ui_strings.notification_timeout, &s->notification_timeout);
-        config_setting_lookup_int(setting, ui_strings.nodeslist_update_freq, &s->nodeslist_update_freq);
-        config_setting_lookup_int(setting, ui_strings.autosave_freq, &s->autosave_freq);
-
-        if (config_setting_lookup_bool(setting, ui_strings.line_padding, &bool_val)) {
-            s->line_padding = bool_val != 0;
-        }
-
-        if (config_setting_lookup_string(setting, ui_strings.line_join, &str)) {
-            snprintf(s->line_join, sizeof(s->line_join), "%s", str);
-        }
-
-        if (config_setting_lookup_string(setting, ui_strings.line_quit, &str)) {
-            snprintf(s->line_quit, sizeof(s->line_quit), "%s", str);
-        }
-
-        if (config_setting_lookup_string(setting, ui_strings.line_alert, &str)) {
-            snprintf(s->line_alert, sizeof(s->line_alert), "%s", str);
-        }
-
-        if (config_setting_lookup_string(setting, ui_strings.line_normal, &str)) {
-            snprintf(s->line_normal, sizeof(s->line_normal), "%s", str);
-        }
-
-        if (config_setting_lookup_string(setting, ui_strings.line_special, &str)) {
-            snprintf(s->line_special, sizeof(s->line_special), "%s", str);
-        }
-
-        if (config_setting_lookup_bool(setting, ui_strings.mplex_away, &bool_val)) {
-            s->mplex_away = bool_val != 0;
-        }
-
-        if (config_setting_lookup_string(setting, ui_strings.mplex_away_note, &str)) {
-            snprintf(s->mplex_away_note, sizeof(s->mplex_away_note), "%s", str);
-        }
-
-        if (config_setting_lookup_string(setting, ui_strings.group_part_message, &str)) {
-            snprintf(s->group_part_message, sizeof(s->group_part_message), "%s", str);
+    if (config_setting_lookup_int(setting, ui_strings.time_format, &time)) {
+        if (time == 12) {
+            snprintf(s->timestamp_format, sizeof(s->timestamp_format), "%s", "%I:%M %p");
+            snprintf(s->log_timestamp_format, sizeof(s->log_timestamp_format), "%s", "%Y/%m/%d [%I:%M %p]");
         }
     }
 
-    /* paths */
-    if ((setting = config_lookup(cfg, tox_strings.self)) != NULL) {
-        if (config_setting_lookup_string(setting, tox_strings.download_path, &str)) {
-            snprintf(s->download_path, sizeof(s->download_path), "%s", str);
-            const size_t len = strlen(s->download_path);
+    if (config_setting_lookup_string(setting, ui_strings.timestamp_format, &str)) {
+        snprintf(s->timestamp_format, sizeof(s->timestamp_format), "%s", str);
+    }
 
-            /* make sure path ends with a '/' */
-            if (len >= sizeof(s->download_path) - 2) {
-                s->download_path[0] = '\0';
-            } else if (len > 0 && s->download_path[len - 1] != '/') {
-                strcat(&s->download_path[len - 1], "/");
-            }
+    if (config_setting_lookup_string(setting, ui_strings.color_bar_bg, &str)) {
+        snprintf(s->color_bar_bg, sizeof(s->color_bar_bg), "%s", str);
+    }
+
+    if (config_setting_lookup_string(setting, ui_strings.color_bar_fg, &str)) {
+        snprintf(s->color_bar_fg, sizeof(s->color_bar_fg), "%s", str);
+    }
+
+    if (config_setting_lookup_string(setting, ui_strings.color_bar_accent, &str)) {
+        snprintf(s->color_bar_accent, sizeof(s->color_bar_accent), "%s", str);
+    }
+
+    if (config_setting_lookup_string(setting, ui_strings.color_bar_notify, &str)) {
+        snprintf(s->color_bar_notify, sizeof(s->color_bar_notify), "%s", str);
+    }
+
+    if (config_setting_lookup_string(setting, ui_strings.log_timestamp_format, &str)) {
+        snprintf(s->log_timestamp_format, sizeof(s->log_timestamp_format), "%s", str);
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.alerts, &bool_val)) {
+        s->alerts = bool_val != 0;
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.show_notification_content, &bool_val)) {
+        s->show_notification_content = bool_val != 0;
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.bell_on_message, &bool_val)) {
+        s->bell_on_message = bool_val != 0 ? NT_BEEP : 0;
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.bell_on_filetrans, &bool_val)) {
+        s->bell_on_filetrans = bool_val != 0 ? NT_BEEP : 0;
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.bell_on_filetrans_accept, &bool_val)) {
+        s->bell_on_filetrans_accept = bool_val != 0 ? NT_BEEP : 0;
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.bell_on_invite, &bool_val)) {
+        s->bell_on_invite = bool_val != 0 ? NT_BEEP : 0;
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.autolog, &bool_val)) {
+        s->autolog = bool_val != 0;
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.native_colors, &bool_val)) {
+        s->native_colors = bool_val != 0;
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.show_typing_self, &bool_val)) {
+        s->show_typing_self = bool_val != 0;
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.show_typing_other, &bool_val)) {
+        s->show_typing_other = bool_val != 0;
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.show_welcome_msg, &bool_val)) {
+        s->show_welcome_msg = bool_val != 0;
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.show_connection_msg, &bool_val)) {
+        s->show_connection_msg = bool_val != 0;
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.show_group_connection_msg, &bool_val)) {
+        s->show_group_connection_msg = bool_val != 0;
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.show_network_info, &bool_val)) {
+        s->show_network_info = bool_val != 0;
+    }
+
+    config_setting_lookup_int(setting, ui_strings.history_size, &s->history_size);
+    config_setting_lookup_int(setting, ui_strings.notification_timeout, &s->notification_timeout);
+    config_setting_lookup_int(setting, ui_strings.nodeslist_update_freq, &s->nodeslist_update_freq);
+    config_setting_lookup_int(setting, ui_strings.autosave_freq, &s->autosave_freq);
+    config_setting_lookup_int(setting, ui_strings.device_cooldown, &s->device_cooldown);
+
+    if (config_setting_lookup_bool(setting, ui_strings.line_padding, &bool_val)) {
+        s->line_padding = bool_val != 0;
+    }
+
+    if (config_setting_lookup_string(setting, ui_strings.line_join, &str)) {
+        snprintf(s->line_join, sizeof(s->line_join), "%s", str);
+    }
+
+    if (config_setting_lookup_string(setting, ui_strings.line_quit, &str)) {
+        snprintf(s->line_quit, sizeof(s->line_quit), "%s", str);
+    }
+
+    if (config_setting_lookup_string(setting, ui_strings.line_alert, &str)) {
+        snprintf(s->line_alert, sizeof(s->line_alert), "%s", str);
+    }
+
+    if (config_setting_lookup_string(setting, ui_strings.line_normal, &str)) {
+        snprintf(s->line_normal, sizeof(s->line_normal), "%s", str);
+    }
+
+    if (config_setting_lookup_string(setting, ui_strings.line_special, &str)) {
+        snprintf(s->line_special, sizeof(s->line_special), "%s", str);
+    }
+
+    if (config_setting_lookup_bool(setting, ui_strings.mplex_away, &bool_val)) {
+        s->mplex_away = bool_val != 0;
+    }
+
+    if (config_setting_lookup_string(setting, ui_strings.mplex_away_note, &str)) {
+        snprintf(s->mplex_away_note, sizeof(s->mplex_away_note), "%s", str);
+    }
+
+    if (config_setting_lookup_string(setting, ui_strings.group_part_message, &str)) {
+        snprintf(s->group_part_message, sizeof(s->group_part_message), "%s", str);
+    }
+}
+
+static void settings_load_paths(config_t *cfg, Client_Config *s)
+{
+    config_setting_t *setting = config_lookup(cfg, tox_strings.self);
+    const char *str = NULL;
+
+    if (setting == NULL) {
+        return;
+    }
+
+    if (config_setting_lookup_string(setting, tox_strings.download_path, &str)) {
+        snprintf(s->download_path, sizeof(s->download_path), "%s", str);
+        const size_t len = strlen(s->download_path);
+
+        /* make sure path ends with a '/' */
+        if (len >= sizeof(s->download_path) - 2) {
+            s->download_path[0] = '\0';
+        } else if (len > 0 && s->download_path[len - 1] != '/') {
+            s->download_path[len] = '/';
+            s->download_path[len + 1] = '\0';
         }
+    }
 
-        if (config_setting_lookup_string(setting, tox_strings.chatlogs_path, &str)) {
-            snprintf(s->chatlogs_path, sizeof(s->chatlogs_path), "%s", str);
-            const size_t len = strlen(s->chatlogs_path);
+    if (config_setting_lookup_string(setting, tox_strings.chatlogs_path, &str)) {
+        snprintf(s->chatlogs_path, sizeof(s->chatlogs_path), "%s", str);
+        const size_t len = strlen(s->chatlogs_path);
 
-            if (len >= sizeof(s->chatlogs_path) - 2) {
-                s->chatlogs_path[0] = '\0';
-            } else if (len > 0 && s->chatlogs_path[len - 1] != '/') {
-                strcat(&s->chatlogs_path[len - 1], "/");
-            }
+        if (len >= sizeof(s->chatlogs_path) - 2) {
+            s->chatlogs_path[0] = '\0';
+        } else if (len > 0 && s->chatlogs_path[len - 1] != '/') {
+            s->chatlogs_path[len] = '/';
+            s->chatlogs_path[len + 1] = '\0';
         }
+    }
 
-        if (config_setting_lookup_string(setting, tox_strings.avatar_path, &str)) {
-            snprintf(s->avatar_path, sizeof(s->avatar_path), "%s", str);
-            const size_t len = strlen(str);
+    if (config_setting_lookup_string(setting, tox_strings.avatar_path, &str)) {
+        snprintf(s->avatar_path, sizeof(s->avatar_path), "%s", str);
+        const size_t len = strlen(str);
 
-            if (len >= sizeof(s->avatar_path)) {
-                s->avatar_path[0] = '\0';
-            }
+        if (len >= sizeof(s->avatar_path)) {
+            s->avatar_path[0] = '\0';
         }
+    }
 
 #ifdef PYTHON
 
-        if (config_setting_lookup_string(setting, tox_strings.autorun_path, &str)) {
-            snprintf(s->autorun_path, sizeof(s->autorun_path), "%s", str);
-            const size_t len = strlen(str);
+    if (config_setting_lookup_string(setting, tox_strings.autorun_path, &str)) {
+        snprintf(s->autorun_path, sizeof(s->autorun_path), "%s", str);
+        const size_t len = strlen(str);
 
-            if (len >= sizeof(s->autorun_path) - 2) {
-                s->autorun_path[0] = '\0';
-            } else if (len > 0 && s->autorun_path[len - 1] != '/') {
-                strcat(&s->autorun_path[len - 1], "/");
-            }
+        if (len >= sizeof(s->autorun_path) - 2) {
+            s->autorun_path[0] = '\0';
+        } else if (len > 0 && s->autorun_path[len - 1] != '/') {
+            s->autorun_path[len] = '/';
+            s->autorun_path[len + 1] = '\0';
         }
+    }
 
 #endif
 
-        if (config_setting_lookup_string(setting, tox_strings.password_eval, &str)) {
-            snprintf(s->password_eval, sizeof(s->password_eval), "%s", str);
-            const size_t len = strlen(str);
+    if (config_setting_lookup_string(setting, tox_strings.password_eval, &str)) {
+        snprintf(s->password_eval, sizeof(s->password_eval), "%s", str);
+        const size_t len = strlen(str);
 
-            if (len >= sizeof(s->password_eval)) {
-                s->password_eval[0] = '\0';
-            }
+        if (len >= sizeof(s->password_eval)) {
+            s->password_eval[0] = '\0';
         }
     }
+}
 
-    /* keys */
-    if ((setting = config_lookup(cfg, key_strings.self)) != NULL) {
-        const char *tmp = NULL;
+static void settings_load_keys(config_t *cfg, Client_Config *s)
+{
+    config_setting_t *setting = config_lookup(cfg, key_strings.self);
+    const char *tmp = NULL;
 
-        if (config_setting_lookup_string(setting, key_strings.next_tab, &tmp)) {
-            set_key_binding(&s->key_next_tab, &tmp);
-        }
-
-        if (config_setting_lookup_string(setting, key_strings.prev_tab, &tmp)) {
-            set_key_binding(&s->key_prev_tab, &tmp);
-        }
-
-        if (config_setting_lookup_string(setting, key_strings.scroll_line_up, &tmp)) {
-            set_key_binding(&s->key_scroll_line_up, &tmp);
-        }
-
-        if (config_setting_lookup_string(setting, key_strings.scroll_line_down, &tmp)) {
-            set_key_binding(&s->key_scroll_line_down, &tmp);
-        }
-
-        if (config_setting_lookup_string(setting, key_strings.half_page_up, &tmp)) {
-            set_key_binding(&s->key_half_page_up, &tmp);
-        }
-
-        if (config_setting_lookup_string(setting, key_strings.half_page_down, &tmp)) {
-            set_key_binding(&s->key_half_page_down, &tmp);
-        }
-
-        if (config_setting_lookup_string(setting, key_strings.page_bottom, &tmp)) {
-            set_key_binding(&s->key_page_bottom, &tmp);
-        }
-
-        if (config_setting_lookup_string(setting, key_strings.toggle_peerlist, &tmp)) {
-            set_key_binding(&s->key_toggle_peerlist, &tmp);
-        }
-
-        if (config_setting_lookup_string(setting, key_strings.toggle_pastemode, &tmp)) {
-            set_key_binding(&s->key_toggle_pastemode, &tmp);
-        }
-
-        if (config_setting_lookup_string(setting, key_strings.reload_config, &tmp)) {
-            set_key_binding(&s->key_reload_config, &tmp);
-        }
+    if (setting == NULL) {
+        return;
     }
 
+    if (config_setting_lookup_string(setting, key_strings.next_tab, &tmp)) {
+        set_key_binding(&s->key_next_tab, &tmp);
+    }
+
+    if (config_setting_lookup_string(setting, key_strings.prev_tab, &tmp)) {
+        set_key_binding(&s->key_prev_tab, &tmp);
+    }
+
+    if (config_setting_lookup_string(setting, key_strings.scroll_line_up, &tmp)) {
+        set_key_binding(&s->key_scroll_line_up, &tmp);
+    }
+
+    if (config_setting_lookup_string(setting, key_strings.scroll_line_down, &tmp)) {
+        set_key_binding(&s->key_scroll_line_down, &tmp);
+    }
+
+    if (config_setting_lookup_string(setting, key_strings.half_page_up, &tmp)) {
+        set_key_binding(&s->key_half_page_up, &tmp);
+    }
+
+    if (config_setting_lookup_string(setting, key_strings.half_page_down, &tmp)) {
+        set_key_binding(&s->key_half_page_down, &tmp);
+    }
+
+    if (config_setting_lookup_string(setting, key_strings.page_bottom, &tmp)) {
+        set_key_binding(&s->key_page_bottom, &tmp);
+    }
+
+    if (config_setting_lookup_string(setting, key_strings.toggle_peerlist, &tmp)) {
+        set_key_binding(&s->key_toggle_peerlist, &tmp);
+    }
+
+    if (config_setting_lookup_string(setting, key_strings.toggle_pastemode, &tmp)) {
+        set_key_binding(&s->key_toggle_pastemode, &tmp);
+    }
+
+    if (config_setting_lookup_string(setting, key_strings.reload_config, &tmp)) {
+        set_key_binding(&s->key_reload_config, &tmp);
+    }
+}
+
 #ifdef AUDIO
+static void settings_load_audio(config_t *cfg, Client_Config *s)
+{
+    config_setting_t *setting = config_lookup(cfg, audio_strings.self);
+    int bool_val;
 
-    /* Audio */
-    if ((setting = config_lookup(cfg, audio_strings.self)) != NULL) {
-        config_setting_lookup_int(setting, audio_strings.input_device, &s->audio_in_dev);
-        s->audio_in_dev = s->audio_in_dev < 0 || s->audio_in_dev > MAX_DEVICES ? 0 : s->audio_in_dev;
-
-        config_setting_lookup_int(setting, audio_strings.output_device, &s->audio_out_dev);
-        s->audio_out_dev = s->audio_out_dev < 0 || s->audio_out_dev > MAX_DEVICES ? 0 : s->audio_out_dev;
-
-        config_setting_lookup_float(setting, audio_strings.VAD_threshold, &s->VAD_threshold);
-
-        config_setting_lookup_int(setting, audio_strings.conference_audio_channels, &s->conference_audio_channels);
-        s->conference_audio_channels = s->conference_audio_channels <= 0
-                                       || s->conference_audio_channels > 2 ? 1 : s->conference_audio_channels;
-
-        config_setting_lookup_int(setting, audio_strings.chat_audio_channels, &s->chat_audio_channels);
-        s->chat_audio_channels = s->chat_audio_channels <= 0 || s->chat_audio_channels > 2 ? 2 : s->chat_audio_channels;
-
-        if (config_setting_lookup_bool(setting, audio_strings.push_to_talk, &bool_val)) {
-            s->push_to_talk = bool_val != 0;
-        }
+    if (setting == NULL) {
+        return;
     }
 
+    config_setting_lookup_int(setting, audio_strings.input_device, &s->audio_in_dev);
+    s->audio_in_dev = s->audio_in_dev < 0 || s->audio_in_dev > MAX_DEVICES ? 0 : s->audio_in_dev;
+
+    config_setting_lookup_int(setting, audio_strings.output_device, &s->audio_out_dev);
+    s->audio_out_dev = s->audio_out_dev < 0 || s->audio_out_dev > MAX_DEVICES ? 0 : s->audio_out_dev;
+
+    config_setting_lookup_float(setting, audio_strings.VAD_threshold, &s->VAD_threshold);
+
+    config_setting_lookup_int(setting, audio_strings.conference_audio_channels, &s->conference_audio_channels);
+    s->conference_audio_channels = s->conference_audio_channels <= 0
+                                   || s->conference_audio_channels > 2 ? 1 : s->conference_audio_channels;
+
+    config_setting_lookup_int(setting, audio_strings.chat_audio_channels, &s->chat_audio_channels);
+    s->chat_audio_channels = s->chat_audio_channels <= 0 || s->chat_audio_channels > 2 ? 2 : s->chat_audio_channels;
+
+    if (config_setting_lookup_bool(setting, audio_strings.push_to_talk, &bool_val)) {
+        s->push_to_talk = bool_val != 0;
+    }
+}
+
 #endif
 
 #ifdef SOUND_NOTIFY
+static void settings_load_sounds(config_t *cfg, Client_Config *s)
+{
+    config_setting_t *setting = config_lookup(cfg, sound_strings.self);
+    const char *str = NULL;
 
-    /* Sound notifications */
-    if ((setting = config_lookup(cfg, sound_strings.self)) != NULL) {
+    if (setting != NULL) {
         if ((config_setting_lookup_string(setting, sound_strings.notif_error, &str) != CONFIG_TRUE) ||
                 !set_sound(notif_error, str)) {
             if (str && strcasecmp(str, NO_SOUND) != 0) {
@@ -1061,7 +1082,41 @@ int settings_load_main(Client_Config *s, const Run_Options *run_opts)
         set_sound(transfer_pending, PACKAGE_DATADIR "/sounds/ToxicTransferStart.wav");
         set_sound(transfer_completed, PACKAGE_DATADIR "/sounds/ToxicTransferComplete.wav");
     }
+}
 
+#endif
+
+int settings_load_main(Client_Config *s, const Run_Options *run_opts)
+{
+    config_t cfg[1];
+    config_init(cfg);
+
+    /* Load default settings */
+    ui_defaults(s);
+    tox_defaults(s);
+    key_defaults(s);
+
+#ifdef AUDIO
+    audio_defaults(s);
+#endif
+
+    const int c_ret = settings_init_config(cfg, run_opts);
+
+    if (c_ret < 0) {
+        config_destroy(cfg);
+        return c_ret;
+    }
+
+    settings_load_ui(cfg, s);
+    settings_load_paths(cfg, s);
+    settings_load_keys(cfg, s);
+
+#ifdef AUDIO
+    settings_load_audio(cfg, s);
+#endif
+
+#ifdef SOUND_NOTIFY
+    settings_load_sounds(cfg, s);
 #endif
 
     config_destroy(cfg);
@@ -1071,6 +1126,7 @@ int settings_load_main(Client_Config *s, const Run_Options *run_opts)
 void settings_reload(Toxic *toxic)
 {
     Client_Config *c_config = toxic->c_config;
+    Client_Data *client_data = &toxic->client_data;
     const Run_Options *run_opts = toxic->run_opts;
     Windows *windows = toxic->windows;
 
@@ -1080,9 +1136,9 @@ void settings_reload(Toxic *toxic)
         fprintf(stderr, "Failed to reload global settings (error %d)\n", ret);
     }
 
-    friend_reset_default_config_settings(c_config);
+    friend_reset_default_config_settings(toxic->friends, c_config);
 
-    ret = settings_load_friends(run_opts);
+    ret = settings_load_friends(toxic->friends, run_opts);
 
     if (ret < 0) {
         fprintf(stderr, "Failed to reload friend settings (error %d)\n", ret);
@@ -1100,13 +1156,19 @@ void settings_reload(Toxic *toxic)
         fprintf(stderr, "Failed to reload group settings (error %d)\n", ret);
     }
 
+    free_ptr_array((void **) client_data->blocked_words);
+    client_data->blocked_words = NULL;
 
-    ret = settings_load_blocked_words(&toxic->client_data, run_opts);
+    ret = settings_load_blocked_words(client_data, run_opts);
 
     if (ret < 0) {
         fprintf(stderr, "Failed to reload blocked words list (error %d)\n", ret);
     }
 
+    if (init_mplex_away_timer(toxic) == -1) {
+        fprintf(stderr, "Failed to initialize mplex auto-away.\n");
+    }
+
     endwin();
     init_term(c_config, NULL, run_opts->default_locale);
     refresh_window_names(toxic);
