@@ -1,6 +1,6 @@
 /*  video_call.h
  *
- *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
+ *  Copyright (C) 2014-2026 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic. Toxic is free software licensed
  *  under the GNU General Public License 3.0.
@@ -16,12 +16,11 @@
 #include "video_device.h"
 
 ToxAV *init_video(Toxic *toxic);
-void terminate_video(void);
+void terminate_video(ToxAV *av, struct CallControl *cc);
 int start_video_transmission(ToxWindow *self, Toxic *toxic, Call *call);
-int stop_video_transmission(Call *call, int friend_number);
-
-void callback_recv_video_starting(uint32_t friend_number);
-void callback_recv_video_end(uint32_t friend_number);
-void callback_video_end(uint32_t friend_number);
+int stop_video_transmission(ToxAV *av, Call *call, int friend_number);
 
+void callback_recv_video_starting(Toxic *toxic, uint32_t friend_number);
+void callback_recv_video_end(Toxic *toxic, uint32_t friend_number);
+void callback_video_end(ToxAV *av, struct CallControl *cc, uint32_t friend_number);
 #endif /* VIDEO_CALL_H */
