@@ -1,6 +1,6 @@
 /*  video_device.c
  *
- *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
+ *  Copyright (C) 2014-2026 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic. Toxic is free software licensed
  *  under the GNU General Public License 3.0.
@@ -172,8 +172,8 @@ VideoDeviceError init_video_devices(Toxic *toxic)
 
     for (; c_size[vdt_input] <= MAX_DEVICES; ++c_size[vdt_input]) {
         int fd;
-        char device_address[] = "/dev/videoXX";
-        snprintf(device_address + 10, sizeof(char) * strlen(device_address) - 10, "%i", c_size[vdt_input]);
+        char device_address[MAX_STR_SIZE];
+        snprintf(device_address, sizeof(device_address), "/dev/video%i", c_size[vdt_input]);
 
         fd = open(device_address, O_RDWR | O_NONBLOCK, 0);
 
@@ -185,28 +185,25 @@ VideoDeviceError init_video_devices(Toxic *toxic)
 
             /* Query V4L for capture capabilities */
             if (-1 != ioctl(fd, VIDIOC_QUERYCAP, &cap)) {
-                video_input_name = (char *)malloc(strlen((const char *)cap.card) + strlen(device_address) + 4);
+                size_t video_input_name_len = strlen((const char *)cap.card) + strlen(device_address) + 4;
+                video_input_name = malloc(video_input_name_len);
 
                 if (video_input_name == NULL) {
                     close(fd);
                     return vde_InternalError;
                 }
 
-                strcpy(video_input_name, (char *)cap.card);
-                strcat(video_input_name, " (");
-                strcat(video_input_name, (char *)device_address);
-                strcat(video_input_name, ")");
+                snprintf(video_input_name, video_input_name_len, "%s (%s)", (char *)cap.card, device_address);
             } else {
-                video_input_name = (char *)malloc(strlen(device_address) + 3);
+                size_t video_input_name_len = strlen(device_address) + 3;
+                video_input_name = malloc(video_input_name_len);
 
                 if (video_input_name == NULL) {
                     close(fd);
                     return vde_InternalError;
                 }
 
-                strcpy(video_input_name, "(");
-                strcat(video_input_name, device_address);
-                strcat(video_input_name, ")");
+                snprintf(video_input_name, video_input_name_len, "(%s)", device_address);
             }
 
             video_devices_names[vdt_input][c_size[vdt_input]] = video_input_name;
@@ -375,8 +372,8 @@ VideoDeviceError open_video_device(VideoDeviceType type, int32_t selection, uint
 
 #else /* not __OSX__ || __APPLE__ */
         /* Open selected device */
-        char device_address[] = "/dev/videoXX";
-        snprintf(device_address + 10, sizeof(device_address) - 10, "%i", selection);
+        char device_address[MAX_STR_SIZE];
+        snprintf(device_address, sizeof(device_address), "/dev/video%i", selection);
 
         device->fd = open(device_address, O_RDWR);
 
@@ -724,7 +721,7 @@ void *video_thread_poll(void *userdata)  // TODO: maybe use thread for every inp
 
                     /* Send frame data to friend through ToxAV */
                     if (device->cb) {
-                        device->cb(toxic, video_width, video_height, y, u, v, device->cb_data);
+                        device->cb(toxic, device->friend_number, video_width, video_height, y, u, v, device->cb_data);
                     }
 
                     /* Convert YUV420 data to BGR */
