@@ -1,6 +1,6 @@
 /*  windows.c
  *
- *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
+ *  Copyright (C) 2014-2026 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic. Toxic is free software licensed
  *  under the GNU General Public License 3.0.
@@ -25,6 +25,10 @@
 #include "toxic.h"
 #include "windows.h"
 
+#ifdef AUDIO
+#include "audio_call.h"
+#endif
+
 #ifdef GAMES
 #include "game_base.h"
 #endif
@@ -116,7 +120,7 @@ void on_friend_name(Tox *tox, uint32_t friendnumber, const uint8_t *string, size
     Toxic *toxic = (Toxic *) userdata;
     Windows *windows = toxic->windows;
 
-    if (friend_config_alias_is_set(friendnumber)) {
+    if (friend_config_alias_is_set(toxic->friends, friendnumber)) {
         return;
     }
 
@@ -151,7 +155,7 @@ void on_friend_status_message(Tox *tox, uint32_t friendnumber, const uint8_t *st
         ToxWindow *w = windows->list[i];
 
         if (w->onStatusMessageChange != NULL) {
-            w->onStatusMessageChange(w, friendnumber, msg, length);
+            w->onStatusMessageChange(w, toxic, friendnumber, msg, length);
         }
     }
 
@@ -290,7 +294,7 @@ void on_file_chunk_request(Tox *tox, uint32_t friendnumber, uint32_t filenumber,
     Toxic *toxic = (Toxic *) userdata;
     Windows *windows = toxic->windows;
 
-    FileTransfer *ft = get_file_transfer_struct(friendnumber, filenumber);
+    FileTransfer *ft = get_file_transfer_struct(toxic->friends, friendnumber, filenumber);
 
     if (ft == NULL) {
         return;
@@ -317,7 +321,7 @@ void on_file_recv_chunk(Tox *tox, uint32_t friendnumber, uint32_t filenumber, ui
     Toxic *toxic = (Toxic *) userdata;
     Windows *windows = toxic->windows;
 
-    const FileTransfer *ft = get_file_transfer_struct(friendnumber, filenumber);
+    const FileTransfer *ft = get_file_transfer_struct(toxic->friends, friendnumber, filenumber);
 
     if (ft == NULL) {
         return;
@@ -339,7 +343,7 @@ void on_file_recv_control(Tox *tox, uint32_t friendnumber, uint32_t filenumber,
     Toxic *toxic = (Toxic *) userdata;
     Windows *windows = toxic->windows;
 
-    FileTransfer *ft = get_file_transfer_struct(friendnumber, filenumber);
+    FileTransfer *ft = get_file_transfer_struct(toxic->friends, friendnumber, filenumber);
 
     if (ft == NULL) {
         return;
@@ -360,11 +364,15 @@ void on_file_recv_control(Tox *tox, uint32_t friendnumber, uint32_t filenumber,
 }
 
 void on_file_recv(Tox *tox, uint32_t friendnumber, uint32_t filenumber, uint32_t kind, uint64_t file_size,
-                  const uint8_t *filename, size_t filename_length, void *userdata)
+                  const uint8_t *string, size_t length, void *userdata)
 {
     Toxic *toxic = (Toxic *) userdata;
     Windows *windows = toxic->windows;
 
+    char filename[MAX_STR_SIZE + 1];
+    length = copy_tox_str(filename, sizeof(filename), (const char *) string, length);
+    filter_string(filename, length, false);
+
     /* We don't care about receiving avatars */
     if (kind != TOX_FILE_KIND_DATA) {
         tox_file_control(tox, friendnumber, filenumber, TOX_FILE_CONTROL_CANCEL, NULL);
@@ -375,8 +383,7 @@ void on_file_recv(Tox *tox, uint32_t friendnumber, uint32_t filenumber, uint32_t
         ToxWindow *w = windows->list[i];
 
         if (w->onFileRecv != NULL) {
-            w->onFileRecv(w, toxic, friendnumber, filenumber, file_size, (const char *) filename,
-                          filename_length);
+            w->onFileRecv(w, toxic, friendnumber, filenumber, file_size, filename, length);
         }
     }
 }
@@ -1514,7 +1521,7 @@ void kill_all_windows(Toxic *toxic)
             }
 
             case WINDOW_TYPE_FRIEND_LIST: {
-                kill_friendlist(w, windows, c_config);
+                kill_friendlist(w, toxic->friends, toxic->blocked, windows, c_config);
                 break;
             }
 
